Attribute VB_Name = "SearchStuff"
' ************************************************
' Search.BAS
'
' Code for list searching example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Global NumItems As Long
Global List() As Long
Global ListTop As New SearchCell
Global ListBottom As SearchCell
Global NumSearches As Long

' ************************************************
' Locate the item using binary search.
' ************************************************
Public Function BinarySearch(target As Long) As Long
Dim min As Long
Dim max As Long
Dim middle As Long

    NumSearches = 0

    ' During the search the target's index will be
    ' between Min and Max: Min <= target index <= Max
    min = 1
    max = NumItems
    Do While min <= max
        NumSearches = NumSearches + 1
        
        middle = (max + min) / 2
        If target = List(middle) Then     ' We have found it!
            BinarySearch = middle
            Exit Function
        ElseIf target < List(middle) Then ' Search the left half.
            max = middle - 1
        Else                              ' Search the right half.
            min = middle + 1
        End If
    Loop
    
    ' If we get here the target is not in the list.
    BinarySearch = 0
End Function

' ************************************************
' Locate the item using interpolation search.
' ************************************************
Public Function InterpSearch(target As Long) As Long
Dim min As Long
Dim max As Long
Dim middle As Long

    NumSearches = 0
    
    min = 1
    max = NumItems
    Do While min <= max
        ' Prevent division by zero.
        If List(min) = List(max) Then
            ' This must be the item (if it's in the list).
            If List(min) = target Then
                InterpSearch = min
            Else
                InterpSearch = 0
            End If
            Exit Function
        End If

        ' Compute the dividing point.
        middle = min + (target - List(min)) * _
            ((max - min) / (List(max) - List(min)))

        ' Make sure we stay in bounds.
        If middle < min Or middle > max Then
            ' It's not in the list.
            InterpSearch = 0
            Exit Function
        End If

        NumSearches = NumSearches + 1
        If target = List(middle) Then       ' We found it.
            InterpSearch = middle
            Exit Function
        ElseIf target < List(middle) Then   ' Search the left half.
            max = middle - 1
        Else                                ' Search the right half.
            min = middle + 1
        End If
    Loop

    ' If we got to this point, the item is not in the list.
    InterpSearch = 0
End Function

' ************************************************
' Locate the item using exhaustive search.
' ************************************************
Public Function LinearSearch(target As Long) As Long
Dim i As Long

    NumSearches = 0
    
    For i = 1 To NumItems
        NumSearches = NumSearches + 1
        If List(i) >= target Then Exit For
    Next i
    
    If i > NumItems Then
        LinearSearch = 0    ' Not found.
    ElseIf List(i) <> target Then
        LinearSearch = 0    ' Not found.
    Else
        LinearSearch = i    ' Found.
    End If
End Function

' ************************************************
' Locate the item in the linked list using
' exhaustive search.
' ************************************************
Public Function LListSearch(target As Long) As SearchCell
Dim cell As SearchCell

    NumSearches = 0
    
    Set cell = ListTop.NextCell
    Do While Not (cell Is Nothing)
        NumSearches = NumSearches + 1

        If cell.Value >= target Then Exit Do
        Set cell = cell.NextCell
    Loop
    
    If Not (cell Is Nothing) Then
        If cell.Value = target Then
            Set LListSearch = cell
        End If
    End If
End Function

' ************************************************
' Locate the item in the linked list using
' exhaustive search with a bottom sentinel.
' ************************************************
Public Function SentinelSearch(target As Long) As SearchCell
Dim cell As SearchCell
Dim sentinel As New SearchCell

    NumSearches = 0

    ' Initialize the sentinel.
    sentinel.Value = target
    Set ListBottom.NextCell = sentinel
    
    ' Find the target.
    Set cell = ListTop.NextCell
    Do While cell.Value < target
        NumSearches = NumSearches + 1
        Set cell = cell.NextCell
    Loop

    ' See if we found it.
    If Not ((cell Is sentinel) Or _
        (cell.Value <> target)) _
    Then
        Set SentinelSearch = cell   ' Found it.
    End If

    ' Remove the sentinel.
    Set ListBottom.NextCell = Nothing
End Function

