VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "BucketHashTable"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' HashBck2.CLS
'
' Hash table using buckets on disk class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Public NumBuckets As Integer    ' # of normal buckets.
Public MaxOverflow As Integer   ' Index of last overflow bucket.

Private TheBucket As BucketType ' The bucket.
Private DataFile As Integer     ' The data file number.

' ************************************************
' Get a bucket.
' ************************************************
Private Sub GetBucket(num As Integer)
    Get #DataFile, num + 1, TheBucket
End Sub
' ************************************************
' Save a bucket.
' ************************************************
Private Sub PutBucket(num As Integer)
    Put #DataFile, num + 1, TheBucket
End Sub
' ************************************************
' Display a textual representation of the table.
' ************************************************
Public Sub DisplayTable(Value As Long, ctl As TextBox)
Dim txt As String
Dim i As Integer
Dim j As Integer
Dim pos As Integer

    txt = ""
    For i = 0 To MaxOverflow
        If i = NumBuckets Then txt = txt & vbCrLf & "*** OVERFLOW ***" & vbCrLf & vbCrLf
        txt = txt & Format$(i) & ": "
        
        ' Get the bucket.
        GetBucket i
        
        For j = 0 To MAX_ITEM
            If TheBucket.Item(j).Value = UNUSED Then
                txt = txt & "---- "
            Else
                txt = txt & Format$(TheBucket.Item(j).Value, "@@@@") & " "
            End If
        Next j
        txt = txt & vbCrLf
    Next i

    ctl.Text = txt

    ' Highlight the selected value.
    pos = InStr(txt, " " & Format$(Value) & " ")
    If pos = 0 Then
        ctl.SelLength = 0
    Else
        ctl.SetFocus
        ctl.SelStart = pos
        ctl.SelLength = Len(Format$(Value))
    End If
End Sub
' ************************************************
' Create a new hash table in a file.
' ************************************************
Public Sub CreateTable(num_buckets As Integer, filename As String)
Dim i As Integer

    NumBuckets = num_buckets
    MaxOverflow = NumBuckets - 1
    
    DataFile = FreeFile
    Open filename For Random As #DataFile Len = BUCKET_SIZE

    ' Initialize an empty bucket.
    For i = 0 To MAX_ITEM
        TheBucket.Item(i).Value = UNUSED
    Next i
    
    ' Save the empty buckets to the file.
    For i = 0 To NumBuckets - 1
        PutBucket i
    Next i
End Sub
' ************************************************
' Close the hash table file.
' ************************************************
Public Sub CloseFile()
    Close DataFile
End Sub

' ************************************************
' Insert the item. Return:
'   HASH_FOUND      The item is already there
'   HASH_INSERTED   The item was inserted
' ************************************************
Public Function InsertItem(Value As Long, bucket_probes As Integer, item_probes As Integer) As Integer
Dim bucket As Integer
Dim pos As Integer
    
    ' First see if the item is already present.
    If LocateItem(Value, bucket_probes, item_probes) _
        = HASH_FOUND _
    Then
        InsertItem = HASH_FOUND
        Exit Function
    End If
    
    ' Get the bucket it should be in.
    bucket_probes = bucket_probes + 1
    bucket = Value Mod NumBuckets
    GetBucket bucket

    ' See if the last position in the bucket is used.
    item_probes = item_probes + 1
    If TheBucket.Item(MAX_ITEM).Value <> UNUSED Then
        ' Use the last overflow bucket.
        bucket_probes = bucket_probes + 1
        bucket = MaxOverflow
        GetBucket bucket
        
        ' See if the last overflow bucket is full.
        item_probes = item_probes + 1
        If TheBucket.Item(MAX_ITEM).Value <> UNUSED Then
            ' It's full. Make a new bucket.
            MaxOverflow = MaxOverflow + 1
            
            item_probes = item_probes + ITEMS_PER_BUCKET
            For pos = 0 To MAX_ITEM
                TheBucket.Item(pos).Value = UNUSED
            Next pos
            bucket = MaxOverflow
        End If
    End If
    
    ' At this point bucket is the index of a
    ' bucket with at least one unused entry.
    ' Find the first unused entry.
    For pos = 0 To MAX_ITEM
        item_probes = item_probes + 1
        If TheBucket.Item(pos).Value = UNUSED Then Exit For
    Next pos

    ' Finally insert the item.
    TheBucket.Item(pos).Value = Value
    bucket_probes = bucket_probes + 1
    PutBucket bucket
    InsertItem = HASH_INSERTED
End Function
' ************************************************
' Locate the item. Return:
'   HASH_FOUND      The item was found.
'   HASH_NOT_FOUND  The item is not in the table.
' ************************************************
Public Function LocateItem(Value As Long, bucket_probes As Integer, item_probes As Integer) As Integer
Dim bucket As Integer
Dim pos As Integer

    item_probes = 0

    ' See what bucket it belongs in.
    GetBucket Value Mod NumBuckets
    bucket_probes = 1
    
    ' Look for the item or an unused position.
    For pos = 0 To MAX_ITEM
        item_probes = item_probes + 1
        If TheBucket.Item(pos).Value = UNUSED Then
            LocateItem = HASH_NOT_FOUND ' Not here.
            Exit Function
        End If
        If TheBucket.Item(pos).Value = Value Then
            LocateItem = HASH_FOUND     ' We found it.
            Exit Function
        End If
    Next pos
    
    ' Check the overflow buckets
    For bucket = NumBuckets To MaxOverflow
        ' Get the next overflow bucket.
        GetBucket bucket
        bucket_probes = bucket_probes + 1
        
        For pos = 0 To MAX_ITEM
            item_probes = item_probes + 1
            If TheBucket.Item(pos).Value = UNUSED Then
                LocateItem = HASH_NOT_FOUND ' Not here.
                Exit Function
            End If
            If TheBucket.Item(pos).Value = Value Then
                LocateItem = HASH_FOUND     ' We found it.
                Exit Function
            End If
        Next pos
    Next bucket
    
    ' If we still haven't found it, it's not here.
    LocateItem = HASH_NOT_FOUND
End Function

