VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "BucketHashTable"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' HashBuck.CLS
'
' Hash table using buckets class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Public NumBuckets As Integer    ' # of normal buckets.
Public MaxOverflow As Integer   ' Index of last overflow bucket.
Public BucketSize As Integer    ' Items per bucket.

Private Buckets() As ItemType ' The buckets.
' ************************************************
' Display a textual representation of the table.
' ************************************************
Public Sub DisplayTable(Value As Long, ctl As TextBox)
Dim txt As String
Dim i As Integer
Dim j As Integer
Dim pos As Integer

    txt = ""
    For i = 0 To MaxOverflow
        If i = NumBuckets Then txt = txt & vbCrLf & "*** OVERFLOW ***" & vbCrLf & vbCrLf
        txt = txt & Format$(i) & ": "
        For j = 0 To BucketSize - 1
            If Buckets(j, i).Value = UNUSED Then
                txt = txt & "---- "
            Else
                txt = txt & Format$(Buckets(j, i).Value, "@@@@") & " "
            End If
        Next j
        txt = txt & vbCrLf
    Next i

    ctl.Text = txt

    ' Highlight the selected value.
    pos = InStr(txt, " " & Format$(Value) & " ")
    If pos = 0 Then
        ctl.SelLength = 0
    Else
        ctl.SetFocus
        ctl.SelStart = pos
        ctl.SelLength = Len(Format$(Value))
    End If
End Sub
' ************************************************
' Create a new hash table.
' ************************************************
Public Sub CreateTable(num_buckets As Integer, bucket_size As Integer)
Dim i As Integer
Dim j As Integer

    NumBuckets = num_buckets
    BucketSize = bucket_size
    ReDim Buckets(0 To BucketSize - 1, 0 To NumBuckets - 1)
    For i = 0 To NumBuckets - 1
        For j = 0 To BucketSize - 1
            Buckets(j, i).Value = UNUSED
        Next j
    Next i
    MaxOverflow = NumBuckets - 1
End Sub
' ************************************************
' Insert the item. Return:
'   HASH_FOUND      The item is already there
'   HASH_INSERTED   The item was inserted
' ************************************************
Public Function InsertItem(Value As Long, bucket_probes As Integer, item_probes As Integer) As Integer
Dim bucket As Integer
Dim pos As Integer
    
    ' First see if the item is already present.
    If LocateItem(Value, bucket_probes, item_probes) _
        = HASH_FOUND _
    Then
        InsertItem = HASH_FOUND
        Exit Function
    End If
    
    ' See what bucket it belongs in.
    bucket = (Value Mod NumBuckets)
    
    ' See if the last position in the bucket is used.
    item_probes = item_probes + 1
    bucket_probes = bucket_probes + 1
    If Buckets(BucketSize - 1, bucket).Value <> UNUSED Then
        ' Use the last overflow bucket.
        bucket = MaxOverflow
        bucket_probes = bucket_probes + 1
        
        ' See if the last overflow bucket is full.
        item_probes = item_probes + 1
        If Buckets(BucketSize - 1, bucket).Value <> UNUSED Then
            ' It's full. Make a new bucket.
            bucket_probes = bucket_probes + 1
            item_probes = item_probes + BucketSize
            MaxOverflow = MaxOverflow + 1
            ReDim Preserve Buckets(0 To BucketSize - 1, 0 To MaxOverflow)
            For pos = 0 To BucketSize - 1
                Buckets(pos, MaxOverflow).Value = UNUSED
            Next pos
            bucket = MaxOverflow
        End If
    End If
    
    ' At this point bucket is the index of a
    ' bucket with at least one unused entry.
    ' Find the first unused entry.
    For pos = 0 To BucketSize - 1
        item_probes = item_probes + 1
        If Buckets(pos, bucket).Value = UNUSED Then Exit For
    Next pos

    ' Finally insert the item.
    Buckets(pos, bucket).Value = Value
    InsertItem = HASH_INSERTED
End Function
' ************************************************
' Locate the item. Return:
'   HASH_FOUND      The item was found.
'   HASH_NOT_FOUND  The item is not in the table.
' ************************************************
Public Function LocateItem(Value As Long, bucket_probes As Integer, item_probes As Integer) As Integer
Dim bucket As Integer
Dim pos As Integer

    bucket_probes = 1
    item_probes = 0

    ' See what bucket it belongs in.
    bucket = (Value Mod NumBuckets)
    
    ' Look for the item or an unused position.
    For pos = 0 To BucketSize - 1
        item_probes = item_probes + 1
        If Buckets(pos, bucket).Value = UNUSED Then
            LocateItem = HASH_NOT_FOUND ' Not here.
            Exit Function
        End If
        If Buckets(pos, bucket).Value = Value Then
            LocateItem = HASH_FOUND     ' We found it.
            Exit Function
        End If
    Next pos
    
    ' Check the overflow buckets
    For bucket = NumBuckets To MaxOverflow
        bucket_probes = bucket_probes + 1
        For pos = 0 To BucketSize - 1
            item_probes = item_probes + 1
            If Buckets(pos, bucket).Value = UNUSED Then
                LocateItem = HASH_NOT_FOUND ' Not here.
                Exit Function
            End If
            If Buckets(pos, bucket).Value = Value Then
                LocateItem = HASH_FOUND     ' We found it.
                Exit Function
            End If
        Next pos
    Next bucket
    
    ' If we still haven't found it, it's not here.
    LocateItem = HASH_NOT_FOUND
End Function

