VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ChainingHashTable"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' HashChain.CLS
'
' Hash table using chaining class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' The list sentinels.
Private m_ListTops() As New ChainCell
Private m_NumLists As Integer ' # of linked lists.

' Indicates whether we should sort the lists.
Public SortLists As Boolean
' ************************************************
' Add an item to the hash table. Return:
'   HASH_FOUND      if the item is already present
'   HASH_INSERTED   if the item was added
' ************************************************
Public Function InsertItem(Value As Long, probes As Integer) As Integer
    If SortLists Then
        InsertItem = InsertItemSorted(Value, probes)
    Else
        InsertItem = InsertItemUnsorted(Value, probes)
    End If
End Function
' ************************************************
' Add an item to the sorted hash table. Return:
'   HASH_FOUND      if the item is already present
'   HASH_INSERTED   if the item was added
' ************************************************
Private Function InsertItemSorted(Value As Long, probes As Integer) As Integer
Dim cell As ChainCell
Dim nxt As ChainCell
Dim new_cell As ChainCell

    probes = 1
    
    ' See where the item belongs in its list.
    Set cell = m_ListTops(Value Mod NumLists)
    Do
        Set nxt = cell.NextCell
        If nxt Is Nothing Then Exit Do
        If nxt.Value >= Value Then Exit Do
        probes = probes + 1
        Set cell = nxt
    Loop
    
    ' If the item is already there, we're done.
    If Not (nxt Is Nothing) Then
        If nxt.Value = Value Then
            InsertItemSorted = HASH_FOUND
            Exit Function
        End If
    End If

    ' Insert the item.
    Set new_cell = New ChainCell
    new_cell.Value = Value
    Set new_cell.NextCell = nxt
    Set cell.NextCell = new_cell
    
    InsertItemSorted = HASH_INSERTED
End Function
' ************************************************
' Add an item to the unsorted hash table. Return:
'   HASH_FOUND      if the item is already present
'   HASH_INSERTED   if the item was added
' ************************************************
Private Function InsertItemUnsorted(Value As Long, probes As Integer) As Integer
Dim cell As ChainCell
Dim new_cell As New ChainCell

    ' See if the item is already present.
    If LocateItemUnsorted(Value, probes) = HASH_FOUND Then
        InsertItemUnsorted = HASH_FOUND
        Exit Function
    End If
    
    ' Get the top of the linked list
    Set cell = m_ListTops(Value Mod NumLists)
    
    ' Insert the item at the top of the list.
    new_cell.Value = Value
    Set new_cell.NextCell = cell.NextCell
    Set cell.NextCell = new_cell

    InsertItemUnsorted = HASH_INSERTED
End Function
' ************************************************
' Locate the item in the sorted list. Return:
'   HASH_FOUND      if the item is present
'   HASH_NOT_FOUND  if the item is not present
' ************************************************
Private Function LocateItemSorted(Value As Long, probes As Integer) As Integer
Dim cell As ChainCell

    probes = 1
    
    ' Get the top of the linked list
    Set cell = m_ListTops(Value Mod NumLists).NextCell
    Do While Not (cell Is Nothing)
        If cell.Value >= Value Then Exit Do
        Set cell = cell.NextCell
        probes = probes + 1
    Loop
    
    If cell Is Nothing Then
        LocateItemSorted = HASH_NOT_FOUND
    ElseIf cell.Value = Value Then
        LocateItemSorted = HASH_FOUND
    Else
        LocateItemSorted = HASH_NOT_FOUND
    End If
End Function
' ************************************************
' Use the appropriate routine to locate the item.
' Return:
'   HASH_FOUND      if the item is present
'   HASH_NOT_FOUND  if the item is not present
' ************************************************
Public Function LocateItem(Value As Long, probes As Integer) As Integer
    If SortLists Then
        LocateItem = LocateItemSorted(Value, probes)
    Else
        LocateItem = LocateItemUnsorted(Value, probes)
    End If
End Function
' ************************************************
' Locate the item in the unsorted list. Return:
'   HASH_FOUND      if the item is present
'   HASH_NOT_FOUND  if the item is not present
' ************************************************
Private Function LocateItemUnsorted(Value As Long, probes As Integer) As Integer
Dim cell As ChainCell

    probes = 1
    
    ' Get the top of the linked list
    Set cell = m_ListTops(Value Mod NumLists).NextCell
    Do While Not (cell Is Nothing)
        If cell.Value = Value Then Exit Do
        Set cell = cell.NextCell
        probes = probes + 1
    Loop
    
    If cell Is Nothing Then
        LocateItemUnsorted = HASH_NOT_FOUND
    Else
        LocateItemUnsorted = HASH_FOUND
    End If
End Function
' ************************************************
' Save the number of lists and allocate the list
' sentinels.
' ************************************************
Property Let NumLists(num_lists As Integer)
    m_NumLists = num_lists
    ReDim m_ListTops(0 To m_NumLists - 1)
End Property
' ************************************************
' Display a textual representation of the table.
' ************************************************
Public Sub DisplayTable(Value As Long, ctl As TextBox)
Dim txt As String
Dim i As Integer
Dim cell As ChainCell
Dim pos As Integer

    For i = 0 To NumLists - 1
        txt = txt & Format$(i, "@@@") & ": "
        Set cell = m_ListTops(i).NextCell
        Do While Not (cell Is Nothing)
            txt = txt & Format$(cell.Value, "@@@") & " "
            Set cell = cell.NextCell
        Loop
        txt = txt & vbCrLf
    Next i
    ctl.Text = txt

    ' Highlight the selected value.
    pos = InStr(txt, " " & Format$(Value) & " ")
    If pos = 0 Then
        ctl.SelLength = 0
    Else
        ctl.SetFocus
        ctl.SelStart = pos
        ctl.SelLength = Len(Format$(Value))
    End If
End Sub
' ************************************************
' Return m_NumLists.
' ************************************************
Property Get NumLists() As Integer
    NumLists = m_NumLists
End Property


