VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "OrderedLinearHashTable"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' HashOrd.CLS
'
' Main form for hashing with ordered linear
' probing example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private m_NumEntries As Long
Private m_NumUnused As Long
Private m_HashTable() As Long
' ************************************************
' Create the new hash table.
' ************************************************
Public Sub CreateTable(num_entries As Integer)
Dim i As Integer

    m_NumEntries = num_entries
    ReDim m_HashTable(0 To m_NumEntries - 1)
    For i = 0 To m_NumEntries - 1
        m_HashTable(i) = UNUSED
    Next i
    m_NumUnused = m_NumEntries
End Sub

' ************************************************
' Insert the item into the table. Return:
'   HASH_FOUND      if the item was already there
'   HASH_INSERTED   if we inserted the item
'   HASH_TABLE_FULL if the table was full
' ************************************************
Public Function InsertItem(ByVal Value As Long, pos As Integer, probes As Integer) As Integer
Dim new_value As Long
Dim status As Integer

    ' See if the table is full.
    If m_NumUnused < 1 Then
        ' See if we can find the item.
        status = LocateItem(Value, pos, probes)
        If status = HASH_FOUND Then
            InsertItem = HASH_FOUND
        Else
            InsertItem = HASH_TABLE_FULL
            pos = -1
        End If
        Exit Function
    End If

    probes = 1
    pos = (Value Mod m_NumEntries)
    Do
        new_value = m_HashTable(pos)

        ' If we found the value, we're done.
        If new_value = Value Then
            InsertItem = HASH_FOUND
            Exit Function
        End If

        ' If the entry is unused, this is where it
        ' belongs.
        If new_value = UNUSED Then
            m_HashTable(pos) = Value
            HashForm.TableControl(pos).Caption = Format$(Value)
            InsertItem = HASH_INSERTED
            m_NumUnused = m_NumUnused - 1
            Exit Function
        End If
        
        ' If the table entry is larger than the
        ' value, swap them and continue.
        If new_value > Value Then
            m_HashTable(pos) = Value
            Value = new_value
        End If
        
        pos = (pos + 1) Mod NumEntries
        probes = probes + 1
    Loop
End Function
' ************************************************
' Find the item in the table. Return:
'   HASH_FOUND      if we find the item
'   HASH_NOT_FOUND  if the item isn't there
' ************************************************
Public Function LocateItem(Value As Long, pos As Integer, probes As Integer) As Integer
Dim new_value As Long

    probes = 1
    pos = (Value Mod m_NumEntries)
    Do
        new_value = m_HashTable(pos)
        
        ' The item's not here.
        If new_value = UNUSED Or probes > NumEntries Then
            LocateItem = HASH_NOT_FOUND
            pos = -1
            Exit Function
        End If
        
        ' We found it or it's not here.
        If new_value >= Value Then Exit Do
        
        pos = (pos + 1) Mod NumEntries
        probes = probes + 1
    Loop

    If Value = new_value Then
        LocateItem = HASH_FOUND
    Else
        LocateItem = HASH_NOT_FOUND
    End If
End Function

' ************************************************
' Return m_NumEntries.
' ************************************************
Property Get NumEntries() As Long
    NumEntries = m_NumEntries
End Property
' ************************************************
' Return m_NumUnused.
' ************************************************
Property Get NumUnused() As Long
    NumUnused = m_NumUnused
End Property

' ************************************************
' Return true if the position is in use.
' ************************************************
Public Function PositionUsed(Index As Integer) As Boolean
    PositionUsed = (m_HashTable(Index) <> UNUSED)
End Function


