VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LinearRehashTable"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' HashRe.CLS
'
' Main form for hashing with linear probing
' example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private m_NumEntries As Long
Private m_NumUnused As Long
Private m_HashTable() As ItemType
' ************************************************
' Create the new hash table.
' ************************************************
Public Sub CreateTable(num_entries As Integer)
Dim i As Integer

    m_NumEntries = num_entries
    ReDim m_HashTable(0 To m_NumEntries - 1)
    For i = 0 To m_NumEntries - 1
        m_HashTable(i).Value = UNUSED
    Next i
    m_NumUnused = m_NumEntries
End Sub

' ************************************************
' Insert the item into the table. Return:
'   HASH_FOUND      if the item was already there
'   HASH_INSERTED   if we inserted the item
'   HASH_TABLE_FULL if the table was full
' ************************************************
Public Function InsertItem(Value As Long, pos As Integer, probes As Integer) As Integer
Dim new_value As Long
Dim status As Integer

    ' See if the table is full.
    If m_NumUnused < 1 Then
        ' See if we can find the item.
        status = LocateItem(Value, pos, probes)
        If status = HASH_FOUND Then
            InsertItem = HASH_FOUND
        Else
            InsertItem = HASH_TABLE_FULL
            pos = -1
        End If
        Exit Function
    End If

    probes = 1
    pos = (Value Mod m_NumEntries)
    Do
        new_value = m_HashTable(pos).Value

        ' If we found the value, we're done.
        If new_value = Value Then
            InsertItem = HASH_FOUND
            Exit Function
        End If

        ' If the entry is unused or deleted,
        ' this is where it belongs.
        If new_value = UNUSED Or new_value = DELETED Then
            m_HashTable(pos).Value = Value
            HashForm.TableControl(pos).Caption = Format$(Value)
            InsertItem = HASH_INSERTED
            m_NumUnused = m_NumUnused - 1
            Exit Function
        End If
        
        pos = (pos + 1) Mod NumEntries
        probes = probes + 1
        
        If probes > NumEntries Then
            ' The entry is not here and the
            ' table is full.
            InsertItem = HASH_TABLE_FULL
            pos = -1
            Exit Function
        End If
    Loop
End Function
' ************************************************
' Remove an item from the table. Return:
'   HASH_NOT_FOUND  if the item is not present
'   HASH_DELETED    if we removed the item
' ************************************************
Public Function DeleteItem(Value As Long, pos As Integer, probes As Integer) As Integer
Dim new_value As Long
Dim status As Integer

    probes = 1
    pos = (Value Mod m_NumEntries)
    Do
        new_value = m_HashTable(pos).Value

        ' If we found the value, mark it as removed.
        If new_value = Value Then
            m_HashTable(pos).Value = DELETED
            DeleteItem = HASH_DELETED
            m_NumUnused = m_NumUnused + 1
            HashForm.TableControl(pos).Caption = "***"
            Exit Function
        End If

        ' If the entry is unused or we have checked
        ' them all, it's not here.
        If new_value = UNUSED Or probes >= m_NumEntries Then
            DeleteItem = HASH_NOT_FOUND
            pos = -1
            Exit Function
        End If
        
        pos = (pos + 1) Mod NumEntries
        probes = probes + 1
    Loop
End Function
' ************************************************
' Find the item in the table. Return:
'   HASH_FOUND      if we find the item
'   HASH_NOT_FOUND  if the item isn't there
' ************************************************
Public Function LocateItem(Value As Long, pos As Integer, probes As Integer) As Integer
Dim new_value As Long

    probes = 1
    pos = (Value Mod m_NumEntries)
    Do
        new_value = m_HashTable(pos).Value
        
        ' We found it.
        If new_value = Value Then
            LocateItem = HASH_FOUND
            Exit Function
        End If
        
        ' The item's not here.
        If new_value = UNUSED Or probes >= NumEntries Then
            LocateItem = HASH_NOT_FOUND
            pos = -1
            Exit Function
        End If

        pos = (pos + 1) Mod NumEntries
        probes = probes + 1
    Loop
End Function

' ************************************************
' Return m_NumEntries.
' ************************************************
Property Get NumEntries() As Long
    NumEntries = m_NumEntries
End Property
' ************************************************
' Return m_NumUnused.
' ************************************************
Property Get NumUnused() As Long
    NumUnused = m_NumUnused
End Property

' ************************************************
' Return true if the position is in use.
' ************************************************
Public Function PositionUsed(Index As Integer) As Boolean
    PositionUsed = (m_HashTable(Index).Value <> UNUSED)
End Function

' ************************************************
' Rehash the table in place.
' ************************************************
Public Sub Rehash()
Dim i As Integer
Dim pos As Integer
Dim probes As Integer
Dim Value As Long
Dim new_value As Long
    
    ' Mark all items as not rehashed.
    For i = 0 To NumEntries - 1
        m_HashTable(i).Rehashed = False
    Next i
    
    ' Look for unrehashed items.
    For i = 0 To NumEntries - 1
        If Not m_HashTable(i).Rehashed Then
            Value = m_HashTable(i).Value
            m_HashTable(i).Value = UNUSED
            HashForm.TableControl(i).Caption = ""
            
            If Value <> DELETED And Value <> UNUSED Then
                ' Follow this item's probe sequence
                ' until we find an empty, deleted,
                ' or unrehashed item.
                probes = 0
                Do
                    pos = (Value + probes) Mod NumEntries
                    new_value = m_HashTable(pos).Value
                    
                    ' If empty or deleted, place
                    ' the item here.
                    If new_value = UNUSED Or _
                       new_value = DELETED _
                    Then
                        m_HashTable(pos).Value = Value
                        m_HashTable(pos).Rehashed = True
                        HashForm.TableControl(pos).Caption = Format$(Value)
                        Exit Do
                    End If
                    
                    ' If not rehashed, swap and
                    ' continue.
                    If Not m_HashTable(pos).Rehashed Then
                        m_HashTable(pos).Value = Value
                        m_HashTable(pos).Rehashed = True
                        HashForm.TableControl(pos).Caption = Format$(Value)
                        Value = new_value
                        probes = 0
                    Else
                        probes = probes + 1
                    End If
                Loop
            End If
        End If
    Next i
End Sub

