VERSION 4.00
Begin VB.Form PathCForm 
   AutoRedraw      =   -1  'True
   Caption         =   "PathC []"
   ClientHeight    =   4140
   ClientLeft      =   1275
   ClientTop       =   1755
   ClientWidth     =   6690
   FillColor       =   &H00FFFFFF&
   FillStyle       =   0  'Solid
   Height          =   4830
   Left            =   1215
   LinkTopic       =   "Form1"
   ScaleHeight     =   276
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   446
   Top             =   1125
   Width           =   6810
   Begin MSComDlg.CommonDialog FileDialog 
      Left            =   120
      Top             =   120
      _ExtentX        =   847
      _ExtentY        =   847
      _Version        =   327681
      CancelError     =   -1  'True
      DefaultExt      =   ".net"
      FileName        =   "*.net"
      Filter          =   "Network Files (*.net)|*.net|All Files (*.*)|*.*"
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileNew 
         Caption         =   "&New"
         Shortcut        =   ^N
      End
      Begin VB.Menu mnuFileOpen 
         Caption         =   "&Open..."
         Shortcut        =   ^O
      End
      Begin VB.Menu mnuFileSave 
         Caption         =   "&Save"
         Shortcut        =   ^S
      End
      Begin VB.Menu mnuFileSaveAs 
         Caption         =   "Save &As..."
         Shortcut        =   ^A
      End
      Begin VB.Menu mnuFileSep 
         Caption         =   "-"
      End
      Begin VB.Menu mnuFileExit 
         Caption         =   "E&xit"
      End
   End
   Begin VB.Menu mnuEdit 
      Caption         =   "&Edit"
      Begin VB.Menu mnuEditMode 
         Caption         =   "New &Node"
         Index           =   1
      End
      Begin VB.Menu mnuEditMode 
         Caption         =   "New &Link"
         Index           =   2
      End
      Begin VB.Menu mnuEditMode 
         Caption         =   "Delete Node"
         Index           =   3
      End
      Begin VB.Menu mnuEditMode 
         Caption         =   "Delete Link"
         Index           =   4
      End
   End
   Begin VB.Menu mnuHelp 
      Caption         =   "&Help"
      Begin VB.Menu mnuHelpAbout 
         Caption         =   "&About This Program..."
      End
   End
End
Attribute VB_Name = "PathCForm"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' PathC.FRM
'
' Main form for label correcting shortest path
' example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Dim Nodes As Collection
Dim Links As Collection
Dim MaxId As Integer    ' The Maximum node ID used.

Dim FileName As String  ' Name of loaded data file.
Dim FileTitle As String ' FileName without path.
Dim Modified As Boolean ' True if data is modified.

Dim EditMode As Integer ' The editing mode.

Const EDIT_MODE_NONE = 0
Const EDIT_MODE_NEW_NODE = 1
Const EDIT_MODE_NEW_LINK = 2
Const EDIT_MODE_NEW_LINK2 = 22
Const EDIT_MODE_DELETE_NODE = 3
Const EDIT_MODE_DELETE_LINK = 4
Const EDIT_MODE_DELETE_LINK2 = 42

' Nodes for manipulating links.
Dim node_1 As PathCNode
Dim node_2 As PathCNode

' Used for drawing and node selection.
Const RADIUS = 10

' Node path tree status values.
Const NOT_IN_LIST = 0
Const NOW_IN_LIST = 1

Const INFINITY = 32767
' ************************************************
' Present help about the program.
' ************************************************
Private Sub mnuHelpAbout_Click()
Dim txt As String

    txt = "This program demonstrates a label correcting shortest path algorithm." & vbCrLf & vbCrLf
    txt = txt & "When you click on a node, the program finds and displays a shortest path tree rooted at that node."
    
    MsgBox txt, vbOKOnly, "Help About ... PathC"
End Sub
' ************************************************
' Create a new link.
' ************************************************
Private Sub CreateNode(X As Single, Y As Single)
Dim node As New PathCNode

    MaxId = MaxId + 1
    node.Id = MaxId
    node.X = X
    node.Y = Y
    Nodes.Add node, Str$(node.Id)

    ' Clear the shortest path tree.
    ResetPathTree
    
    ' Redraw the network.
    DrawNetwork

    ' The data has been modified.
    Modified = True
    Caption = "PathC*[" & FileTitle & "]"
End Sub
' ************************************************
' Return true if the data is safely saved.
' ************************************************
Private Function DataSafe() As Boolean
    If Not Modified Then
        DataSafe = True
        Exit Function
    End If
    
    Select Case _
        MsgBox("Save changes to the network?", _
            vbYesNoCancel, "Save Changes?")
        Case vbYes  ' mnuFileSave will set Modifed.
            mnuFileSave_Click
            DataSafe = (Not Modified)
        Case vbNo   ' It's OK to lose the changes.
            DataSafe = True
        Case vbCancel   ' Stop the exit.
            DataSafe = False
    End Select
End Function

' ************************************************
' Delete the node from the network.
' ************************************************
Private Sub DeleteNode(node As PathCNode)
Dim link As PathCLink

    If node Is Nothing Then Exit Sub

    ' Remove links to this node.
    Do While node.Links.Count > 0
        Set link = node.Links(1)
        DeleteLink link.Node1, link.Node2
    Loop
    
    ' Remove the node from the Nodes list.
    Nodes.Remove Str$(node.Id)

    ' Clear the shortest path tree.
    ResetPathTree
    
    ' Redraw the network.
    DrawNetwork

    ' The data has been modified.
    Modified = True
    Caption = "PathC*[" & FileTitle & "]"
End Sub
' ************************************************
' Display the network.
' ************************************************
Private Sub DrawNetwork()
Dim node As PathCNode
Dim link As PathCLink
Dim node_id As Integer
Dim dx As Single
Dim dy As Single
Dim Dist As Single
Dim x1 As Single
Dim y1 As Single
Dim x2 As Single
Dim y2 As Single
Dim i As Integer
Dim txt As String

    Cls
    
    ' Draw the links.
    FillColor = BackColor
    For Each link In Links
        If link.InPathTree Then
            ' The following two lines seeme to be
            ' needed to make VB5 draw all of these
            ' lines with width 3.
            ForeColor = vbRed
            DrawWidth = 4
            ForeColor = vbBlue
            DrawWidth = 3
            Font.Bold = True
        Else
            ForeColor = vbBlack
            DrawWidth = 1
            Font.Bold = False
        End If
        
        x1 = link.Node1.X
        y1 = link.Node1.Y
        x2 = link.Node2.X
        y2 = link.Node2.Y
        dx = x2 - x1
        dy = y2 - y1
        Dist = Sqr(dx * dx + dy * dy)
        If Dist > 0 Then
            dx = dx * RADIUS / Dist
            dy = dy * RADIUS / Dist
            
            ' Draw the link.
            x1 = x1 + dx
            y1 = y1 + dy
            x2 = x2 - dx
            y2 = y2 - dy
            Line (x1, y1)-(x2, y2)
            
            ' Draw the link cost.
            x1 = (x1 + x2) / 2
            y1 = (y1 + y2) / 2
            
            DrawStyle = vbInvisible
            Circle (x1, y1), RADIUS
            DrawStyle = vbSolid
            
            txt = Format$(link.Cost)
            CurrentX = x1 - TextWidth(txt) / 2
            CurrentY = y1 - TextHeight(txt) / 2
            Print txt
        End If
    Next link
    DrawWidth = 1
    Font.Bold = False
    
    ' Draw the nodes.
    For Each node In Nodes
        ' Draw the node and its links.
        If node Is node_1 Or node Is node_2 Then
            ' Highlight the node.
            FillColor = vbBlack
            ForeColor = vbWhite
        Else
            ' Do not highlight the node.
            FillColor = vbWhite
            ForeColor = vbBlack
        End If
        
        ' Draw the node.
        Circle (node.X, node.Y), RADIUS
        txt = Format$(node.Id)
        CurrentX = node.X - TextWidth(txt) / 2
        CurrentY = node.Y - TextHeight(txt) / 2
        Print txt
    Next node
    ForeColor = vbBlack
End Sub

' ************************************************
' Load the network data from a file with format:
'   # Nodes, # Links
'   For each node: Id, X, Y
'   For each link: Node1, Node2, Cost
' ************************************************
Private Sub LoadNetwork(ByVal fname As String, ByVal ftitle As String)
Dim Id As Integer
Dim X As Single
Dim Y As Single
Dim fnum As Integer
Dim i As Integer
Dim num_nodes As Integer
Dim num_links As Integer
Dim node1_num As Integer
Dim node2_num As Integer
Dim link_cost As Integer
Dim id1 As String
Dim id2 As String
Dim node_1 As PathCNode
Dim node_2 As PathCNode
Dim node As PathCNode
Dim link As PathCLink

    ' Open the file.
    On Error GoTo OpenError
    fnum = FreeFile
    Open fname For Input As fnum

    ' Start a new network.
    NewNetwork
    
    ' Read the number of nodes and links.
    Input #fnum, num_nodes, num_links
    
    ' Read the node identifiers and coordinates.
    MaxId = 0
    For i = 1 To num_nodes
        Set node = New PathCNode
        Input #fnum, Id, X, Y
        node.Id = Id
        node.X = X
        node.Y = Y
        Nodes.Add node, Str$(node.Id)
        If node.Id > MaxId Then MaxId = node.Id
    Next i
    
    ' Read the links.
    For i = 1 To num_links
        Input #fnum, node1_num, node2_num, link_cost
        id1 = Str$(node1_num)
        id2 = Str$(node2_num)
        Set link = New PathCLink
        Links.Add link, id1 & "-" & id2
        Set node_1 = Nodes(id1)
        Set node_2 = Nodes(id2)
        Set link.Node1 = node_1
        Set link.Node2 = node_2
        link.Cost = link_cost
        node_1.Links.Add link, id2
        node_2.Links.Add link, id1
    Next i
    
    ' Close the file.
    On Error GoTo CloseError
    Close #fnum
    
    ' The network has not yet been modified.
    FileName = fname
    FileTitle = ftitle
    Caption = "PathC [" & FileTitle & "]"
    Exit Sub

OpenError:
    Beep
    MsgBox "Error" & Str$(Err.Number) & _
        " opening network file """ & FileName & _
        """." & vbCrLf & Err.Description
    Exit Sub

ReadError:
    Beep
    MsgBox "Error" & Str$(Err.Number) & _
        " reading network file """ & FileName & _
        """." & vbCrLf & Err.Description
    Nodes.Clear
    Exit Sub

CloseError:
    Beep
    MsgBox "Error closing file """ & FileName & _
        """." & vbCrLf & Err.Description
    Exit Sub
End Sub

' ************************************************
' Make a new link.
' ************************************************
Private Sub CreateLink()
Dim link_cost As Integer
Dim link As PathCLink
Dim id1 As String
Dim id2 As String

    If node_1 Is Nothing Or node_2 Is Nothing Then _
        Exit Sub
    
    ' Do not allow links from a node to itself.
    If node_1 Is node_2 Then
        Beep
        MsgBox "You cannot make a link between a node and itself.", _
            vbOKOnly, "Invalid Link"
        Exit Sub
    End If
    
    id1 = Str$(node_1.Id)
    id2 = Str$(node_2.Id)
    
    ' Do not allow duplicate links.
    On Error Resume Next
    Set link = node_1.Links(id2)
    If Err.Number = 0 Then
        On Error GoTo 0
        Beep
        MsgBox "There is already a link between these nodes.", _
            vbOKOnly, "Link Exists"
        Exit Sub
    End If
    On Error GoTo 0

    ' Create the link.
    link_cost = InputBox("Link cost")

    Set link = New PathCLink
    Links.Add link, id1 & "-" & id2
    Set link.Node1 = node_1
    Set link.Node2 = node_2
    link.Cost = link_cost
    node_1.Links.Add link, id2
    node_2.Links.Add link, id1
    
    ' Clear the shortest path tree.
    ResetPathTree
    
    ' Redraw the network.
    DrawNetwork

    ' The data has been modified.
    Modified = True
    Caption = "PathC*[" & FileTitle & "]"
End Sub
' ************************************************
' Delete a link.
' ************************************************
Private Sub DeleteLink(node_1 As PathCNode, node_2 As PathCNode)
Dim id1 As String
Dim id2 As String

    If node_1 Is Nothing Or node_2 Is Nothing Then _
        Exit Sub

    id1 = Str$(node_1.Id)
    id2 = Str$(node_2.Id)
    
    On Error Resume Next
    ' Delete from the nodes' Links collections.
    node_1.Links.Remove id2
    node_2.Links.Remove id1
    
    ' Delete from the global Links collections.
    Links.Remove id1 & "-" & id2
    Links.Remove id2 & "-" & id1
    On Error GoTo 0
    
    ' Clear the shortest path tree.
    ResetPathTree
    
    ' Redraw the network.
    DrawNetwork

    ' The data has been modified.
    Modified = True
    Caption = "PathC*[" & FileTitle & "]"
End Sub
' ************************************************
' Reset the edit mode.
' ************************************************
Private Sub ResetEditMode()
    EditMode = EDIT_MODE_NONE
    MousePointer = vbDefault
    Set node_1 = Nothing
    Set node_2 = Nothing
End Sub

' ************************************************
' Save the network data into a file with format:
'   # Nodes, # Links
'   For each node: Id, X, Y
'   For each link: Node1, Node2, Cost
' ************************************************
Private Sub SaveNetwork(ByVal fname As String, ByVal ftitle As String)
Dim fnum As Integer
Dim node As PathCNode
Dim link As PathCLink

    ' Open the file.
    On Error GoTo OpenError
    fnum = FreeFile
    Open fname For Output As fnum
    
    ' Save the number of nodes and links.
    Write #fnum, Nodes.Count, Links.Count
    
    ' Save the node identifiers and coordinates.
    For Each node In Nodes
        Write #fnum, node.Id, node.X, node.Y
    Next node
    
    ' Save the link information.
    For Each link In Links
        Write #fnum, link.Node1.Id, link.Node2.Id, link.Cost
    Next link
    
    ' Close the file.
    On Error GoTo CloseError
    Close #fnum
    
    Modified = False
    FileName = fname
    FileTitle = ftitle
    Caption = "PathC [" & FileTitle & "]"
    Exit Sub

OpenError:
    Beep
    MsgBox "Error" & Str$(Err.Number) & _
        " opening file """ & FileName & _
        """." & vbCrLf & Err.Description
    Exit Sub

ReadError:
    Beep
    MsgBox "Error" & Str$(Err.Number) & _
        " saving network file """ & FileName & _
        """." & vbCrLf & Err.Description
    Nodes.Clear
    Exit Sub

CloseError:
    Beep
    MsgBox "Error closing file """ & FileName & _
        """." & vbCrLf & Err.Description
    Exit Sub
End Sub
' ************************************************
' Start a new network.
' ************************************************
Private Sub NewNetwork()
Dim node As PathCNode
    
    ' Destroy all links.
    For Each node In Nodes
        Set node.Links = Nothing
    Next node

    ' Create new Nodes and Links collections.
    Set Links = New Collection
    Set Nodes = New Collection
    
    ' The new network has not yet been modified.
    ResetEditMode
    Modified = False
    FileName = ""
    FileTitle = ""
    Caption = "PathC []"
End Sub
' ************************************************
' Find the node clicked.
' ************************************************
Private Function SelectNode(X As Single, Y As Single) As PathCNode
Dim node As PathCNode
Dim dx As Single
Dim dy As Single
Dim radius2 As Single

    radius2 = RADIUS * RADIUS
    For Each node In Nodes
        dx = node.X - X
        dy = node.Y - Y
        If dx * dx + dy * dy <= radius2 Then
            Set SelectNode = node
            Exit Function
        End If
    Next node
End Function
' ************************************************
' Initialize.
' ************************************************
Private Sub Form_Load()
    ' Make the file dialog start in the working
    ' directory.
    FileDialog.InitDir = App.Path
    
    ' Create an empty network.
    Set Nodes = New Collection
    Set Links = New Collection
End Sub

' ************************************************
' Perform an edit mode appropriate task.
' ************************************************
Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Select Case EditMode
        Case EDIT_MODE_NONE     ' Find the shorest path tree.
            FindPathTree SelectNode(X, Y)
            Exit Sub
        Case EDIT_MODE_NEW_NODE ' Put a node here.
            CreateNode X, Y
            EditMode = EDIT_MODE_NONE
        Case EDIT_MODE_NEW_LINK ' Select node 1.
            Set node_1 = SelectNode(X, Y)
            If node_1 Is Nothing Then
                EditMode = EDIT_MODE_NONE
            Else
                EditMode = EDIT_MODE_NEW_LINK2
            End If
        Case EDIT_MODE_NEW_LINK2 ' Make a link.
            Set node_2 = SelectNode(X, Y)
            CreateLink
            EditMode = EDIT_MODE_NONE
        Case EDIT_MODE_DELETE_NODE ' Delete node 1.
            DeleteNode SelectNode(X, Y)
            EditMode = EDIT_MODE_NONE
        Case EDIT_MODE_DELETE_LINK ' Select node 1.
            Set node_1 = SelectNode(X, Y)
            If node_1 Is Nothing Then
                EditMode = EDIT_MODE_NONE
            Else
                EditMode = EDIT_MODE_DELETE_LINK2
            End If
        Case EDIT_MODE_DELETE_LINK2 ' Delete a link.
            Set node_2 = SelectNode(X, Y)
            DeleteLink node_1, node_2
            EditMode = EDIT_MODE_NONE
    End Select

    If EditMode = EDIT_MODE_NONE Then
        MousePointer = vbDefault
        Set node_1 = Nothing
        Set node_2 = Nothing
    End If
    
    DrawNetwork
End Sub
' ************************************************
' Find a shortest path tree rooted at this node
' using a label correcting algorithm.
' ************************************************
Private Sub FindPathTree(root As PathCNode)
Dim candidates As New Collection
Dim node_dist As Integer
Dim new_dist As Integer
Dim node As PathCNode
Dim to_node As PathCNode
Dim link As PathCLink

    If root Is Nothing Then Exit Sub
    
    ' Reset all nodes' Marked and NodeStatus values,
    ' and all links' Used and InPathTree flags.
    ResetPathTree

    ' Start with the root in the shortest path tree.
    root.Dist = 0
    Set root.InLink = Nothing
    root.NodeStatus = NOW_IN_LIST
    candidates.Add root
    
    Do While candidates.Count > 0
        ' Add a node to the shortest path tree.
        Set node = candidates(1)
        candidates.Remove 1
        node_dist = node.Dist
        node.NodeStatus = NOT_IN_LIST
        
        ' Examine the node's neighbors.
        For Each link In node.Links
            If node Is link.Node1 Then
                Set to_node = link.Node2
            Else
                Set to_node = link.Node1
            End If
            
            ' See if there is an improved path
            ' using this node.
            new_dist = node_dist + link.Cost
            If to_node.Dist > new_dist Then
                ' It's better. Update the Dist and
                ' InLink values.
                Set to_node.InLink = link
                to_node.Dist = new_dist
                
                ' Add it to the candidate list if
                ' is not already there.
                If to_node.NodeStatus = NOT_IN_LIST Then
                    candidates.Add to_node
                    to_node.NodeStatus = NOW_IN_LIST
                End If
            End If
        Next link
    Loop
    
    ' Mark the inlinks so they are easy to draw.
    For Each node In Nodes
        If Not (node.InLink Is Nothing) Then _
            node.InLink.InPathTree = True
    Next node
    
    ' Redraw the network.
    DrawNetwork
End Sub
' ************************************************
' Remove all links from the shortest path tree.
' ************************************************
Private Sub ResetPathTree()
Dim node As PathCNode
Dim link As PathCLink

    For Each node In Nodes
        node.Dist = INFINITY
        node.NodeStatus = NOT_IN_LIST
    Next node
    
    For Each link In Links
        link.InPathTree = False
    Next link
End Sub
' ************************************************
' Make sure the data is safe before exiting.
' ************************************************
Private Sub Form_QueryUnload(Cancel As Integer, UnloadMode As Integer)
    Cancel = (Not DataSafe())
End Sub

' ************************************************
' Enter the new editing mode.
' ************************************************
Private Sub mnuEditMode_Click(Index As Integer)
    EditMode = Index
    
    Select Case EditMode
        Case EDIT_MODE_NEW_NODE, _
             EDIT_MODE_DELETE_NODE
                MousePointer = vbCrosshair
        Case EDIT_MODE_NEW_LINK, _
             EDIT_MODE_DELETE_LINK
                MousePointer = vbSizeNESW
    End Select
End Sub

' ************************************************
' End the application. Note that Form_QueryUnload
' ensures that the data is safe.
' ************************************************
Private Sub mnuFileExit_Click()
    Unload Me
End Sub


' ************************************************
' Start a new network.
' ************************************************
Private Sub mnuFileNew_Click()
    ' Make sure the current data is safe.
    If Not DataSafe() Then Exit Sub

    ' Start the new network.
    NewNetwork
    
    ' Display the empty network.
    DrawNetwork
End Sub
' ************************************************
' Load a network file.
' ************************************************
Private Sub mnuFileOpen_Click()
Dim status As Long

    ' Make sure the current data is safe.
    If Not DataSafe() Then Exit Sub
    
    ' Prepare the file selection dialog
    FileDialog.Flags = cdlOFNFileMustExist
    FileDialog.FileName = ""
    
    On Error Resume Next
    FileDialog.ShowOpen
    status = Err
    On Error GoTo 0

    If status = cdlCancel Then Exit Sub
    If status <> 0 Then
        Beep
        MsgBox "Error" & Str$(status) & _
            " selecting file." & vbCrLf & _
            Error(status)
        Exit Sub
    End If

    FileDialog.InitDir = _
        Left$(FileDialog.FileName, _
            Len(FileDialog.FileName) - _
            Len(FileDialog.FileTitle))
    LoadNetwork FileDialog.FileName, FileDialog.FileTitle
    
    ' Display the network.
    DrawNetwork
End Sub

' ************************************************
' Save the file using the current FileName.
' ************************************************
Private Sub mnuFileSave_Click()
    ' Make sure we have a file name.
    If FileName = "" Then
        mnuFileSaveAs_Click
        Exit Sub
    End If
    
    ' Save the network.
    SaveNetwork FileName, FileTitle
End Sub


' ************************************************
' Save the network in a file selected by the user.
' ************************************************
Private Sub mnuFileSaveAs_Click()
Dim status As Long

    ' Prepare the file selection dialog
    FileDialog.Flags = cdlOFNOverwritePrompt
    FileDialog.FileName = FileTitle

    On Error Resume Next
    FileDialog.ShowSave
    status = Err
    On Error GoTo 0

    If status = cdlCancel Then Exit Sub
    If status <> 0 Then
        Beep
        MsgBox "Error" & Str$(status) & _
            " selecting file." & vbCrLf & _
            Error(status)
        Exit Sub
    End If

    FileDialog.InitDir = _
        Left$(FileDialog.FileName, _
            Len(FileDialog.FileName) - _
            Len(FileDialog.FileTitle))
    
    ' Save the network.
    SaveNetwork FileDialog.FileName, FileDialog.FileTitle
End Sub

