VERSION 4.00
Begin VB.Form GraphView 
   Caption         =   "GraphView"
   ClientHeight    =   2730
   ClientLeft      =   2715
   ClientTop       =   1650
   ClientWidth     =   3870
   Height          =   3135
   KeyPreview      =   -1  'True
   Left            =   2655
   LinkTopic       =   "Form1"
   MDIChild        =   -1  'True
   ScaleHeight     =   182
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   258
   Top             =   1305
   Width           =   3990
   Begin VB.PictureBox GraphPict 
      Height          =   1215
      Left            =   1440
      ScaleHeight     =   1155
      ScaleWidth      =   1755
      TabIndex        =   0
      Top             =   840
      Width           =   1815
   End
End
Attribute VB_Name = "GraphView"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' ExpGraph.FRM
'
' Graph view for expense MVC example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Public TheModel As ExpenseModel

Private ViewID As Integer

' Index of the value we're moving.
Private MovingValue As Integer

Private Xmin As Single
Private Xmax As Single
Private RowHgt As Single

Private CurX As Single
Private GapX As Single
Private GapY As Single

' ************************************************
' The data has been modified.
' ************************************************
Public Sub DataModified()
    Refresh
End Sub

' ************************************************
' Save the model and register with it as a view.
' ************************************************
Property Set Model(New_Model As ExpenseModel)
    Set TheModel = New_Model
    ViewID = TheModel.AddView(Me)
    DataModified
    Show
End Property

' ************************************************
' Unload if the user presses the escape key.
' ************************************************
Private Sub Form_KeyPress(KeyAscii As Integer)
    If KeyAscii = vbKeyEscape Then Unload Me
End Sub

Private Sub Form_Load()
    MovingValue = -1
End Sub

' ************************************************
' Display the graph.
' ************************************************
Private Sub Form_Paint()
Const margin = 5

Dim i As Integer
Dim wid As Single
Dim X As Single
Dim Y As Single
Dim txt As String

    Cls
    GraphPict.Cls
    If TheModel.NumValues < 1 Then Exit Sub
    
    ' See which label is the widest.
    Xmin = TextWidth(TheModel.ExpenseName(0))
    For i = 1 To TheModel.NumValues - 1
        wid = TextWidth(TheModel.ExpenseName(i))
        If wid > Xmin Then Xmin = wid
    Next i

    ' Position GraphPict.
    RowHgt = 1.25 * ScaleY(Font.Size, vbPoints, ScaleMode)
    GraphPict.Move Xmin + 2 * margin, margin, _
        ScaleWidth - Xmin - 3 * margin, _
        TheModel.NumValues * RowHgt
    GraphPict.Scale (0, -0.5)-(10000, TheModel.NumValues - 0.5)

    ' Display the labels.
    X = margin
    Y = margin
    For i = 0 To TheModel.NumValues - 1
        txt = TheModel.ExpenseName(i)
        CurrentX = X + Xmin - TextWidth(txt)
        CurrentY = Y
        Print TheModel.ExpenseName(i)
        Y = Y + RowHgt
    Next i

    ' Draw the values.
    Xmin = Xmin + margin
    Xmax = ScaleWidth - margin
    If Xmax <= Xmin Then Exit Sub
    GapX = GraphPict.ScaleX(2, vbPixels, GraphPict.ScaleMode)
    GapY = 0.5
    GraphPict.DrawMode = vbCopyPen
    For i = 0 To TheModel.NumValues - 1
        X = TheModel.ExpenseValue(i)
        GraphPict.Line (0, i - 0.5)-(X, i + 0.5), QBColor(i Mod 16), BF
        GraphPict.Line (0, i - 0.5)-(X, i + 0.5), , B
    Next i
End Sub

Private Sub Form_Resize()
    Form_Paint
End Sub


' ************************************************
' Unregister as a view.
' ************************************************
Private Sub Form_Unload(Cancel As Integer)
    TheModel.RemoveView ViewID
End Sub

' ************************************************
' Start moving a value.
' ************************************************
Private Sub GraphPict_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' See which value it is.
    MovingValue = CInt(Y)
    If MovingValue < 0 Then MovingValue = 0
    If MovingValue >= TheModel.NumValues Then _
        MovingValue = TheModel.NumValues - 1

    ' Make sure the mouse is near the current value.
    CurX = TheModel.ExpenseValue(MovingValue)
    If (X < CurX - GapX) Or (X > CurX + GapX) Then
        MovingValue = -1
        Exit Sub
    End If

    CurX = X
    GraphPict.DrawMode = vbInvert
    GraphPict.Line (CurX - GapX, MovingValue - GapY)-Step(2 * GapX, 2 * GapY), , BF
End Sub

' ************************************************
' Continue moving a value.
' ************************************************
Private Sub GraphPict_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If MovingValue < 0 Then Exit Sub

    GraphPict.DrawMode = vbInvert
    GraphPict.Line (CurX - GapX, MovingValue - GapY)-Step(2 * GapX, 2 * GapY), , BF
    CurX = X
    If CurX < 0 Then CurX = 0
    If CurX > 10000 Then CurX = 10000
    GraphPict.Line (CurX - GapX, MovingValue - GapY)-Step(2 * GapX, 2 * GapY), , BF
End Sub


' ************************************************
' Finish moving a value.
' ************************************************
Private Sub GraphPict_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If MovingValue < 0 Then Exit Sub

    GraphPict.DrawMode = vbInvert
    GraphPict.Line (CurX - GapX, MovingValue - GapY)-Step(2 * GapX, 2 * GapY), , BF
    If X < 0 Then X = 0
    If X > 10000 Then X = 10000
    TheModel.SetExpenseValue Nothing, MovingValue, X
    
    MovingValue = -1
End Sub


