VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ExpenseModel"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' ExpModel.CLS
'
' Expense model for MVC example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private m_NumValues As Integer
Private m_ExpenseValue() As Single
Private m_ExpenseName() As String

' The views.
Private Views As New Collection
Private MaxView As Integer
' ************************************************
' Add a view to the Views collection. Return a
' view ID the program can use to remove the view.
' ************************************************
Public Function AddView(view As Object) As Integer
    MaxView = MaxView + 1
    Views.Add view, Str$(MaxView)
    AddView = MaxView
End Function
' ************************************************
' Notify the views that the data has changed.
' ************************************************
Private Sub NotifyViews(skip As Object)
Dim obj As Object

    For Each obj In Views
        If Not (obj Is skip) Then obj.DataModified
    Next obj
End Sub

' ************************************************
' Remove a view from the Views collection.
' ************************************************
Public Sub RemoveView(id As Integer)
    On Error Resume Next
    Views.Remove Str$(id)
End Sub
' ************************************************
' Return an expense value.
' ************************************************
Property Get ExpenseValue(Index As Integer) As Single
    If Index < 0 Or Index >= m_NumValues Then _
        Err.Raise 381
    
    ExpenseValue = m_ExpenseValue(Index)
End Property
' ************************************************
' Return an expense name.
' ************************************************
Property Get ExpenseName(Index As Integer) As String
    If Index < 0 Or Index >= m_NumValues Then _
        Err.Raise 381

    ExpenseName = m_ExpenseName(Index)
End Property
' ************************************************
' Return the number of expense values.
' ************************************************
Property Get NumValues() As Integer
    NumValues = m_NumValues
End Property
' ************************************************
' Set the number of expense values.
' ************************************************
Public Sub SetNumValues(obj As Object, New_NumValues As Integer)
    If New_NumValues < 1 Then Err.Raise 380 ' Invalid property value.

    m_NumValues = New_NumValues
    ReDim Preserve m_ExpenseValue(0 To m_NumValues - 1)
    ReDim Preserve m_ExpenseName(0 To m_NumValues - 1)

    NotifyViews obj
End Sub
' ************************************************
' Set an expense value.
' ************************************************
Public Sub SetExpenseValue(obj As Object, Index As Integer, New_ExpenseValue As Single)
    If Index < 0 Or Index >= m_NumValues Then _
        Err.Raise 381 ' Invalid property array index.
    
    m_ExpenseValue(Index) = New_ExpenseValue
    NotifyViews obj
End Sub
' ************************************************
' Set an expense name.
' ************************************************
Public Sub SetExpenseName(obj As Object, Index As Integer, New_ExpenseName As String)
    If Index < 0 Or Index >= m_NumValues Then _
        Err.Raise 381 ' Invalid property array index.
    
    m_ExpenseName(Index) = New_ExpenseName
    NotifyViews obj
End Sub

' ************************************************
' Return a sum of the expenses.
' ************************************************
Property Get TotalExpenses() As Single
Dim i As Integer
Dim tot As Single

    For i = 0 To m_NumValues - 1
        tot = tot + m_ExpenseValue(i)
    Next i
    TotalExpenses = tot
End Property


