VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "IterTree"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' IterTree.CLS
'
' Tree class for iterator example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Public TheForm As Form

Private NumNodes As Integer
Private Nodes() As Label
' ************************************************
' Build a full, complete tree of the given depth.
' ************************************************
Public Sub MakeTree(depth As Integer)
Const GAP = 60

Static MaxNode As Integer

Dim wid As Single
Dim hgt As Single
Dim x As Single
Dim y As Single
Dim i As Integer

    ' Compute the number of nodes in the tree
    NumNodes = 1
    For i = 1 To depth
        NumNodes = NumNodes * 2
    Next i
    NumNodes = NumNodes - 1
    ReDim Nodes(0 To NumNodes - 1)

    ' Create the needed controls
    With TheForm
        For i = MaxNode + 1 To NumNodes - 1
            Load .NodeLabel(i)
        Next i
        If MaxNode < NumNodes - 1 Then MaxNode = NumNodes - 1
    
        For i = 0 To NumNodes - 1
            .NodeLabel(i).Visible = True
        Next i
        For i = NumNodes To MaxNode
            .NodeLabel(i).Visible = False
        Next i
        
        ' Arrange the controls
        wid = .NodeLabel(0).Width
        hgt = .NodeLabel(0).Height
        y = (hgt + GAP) * (depth) + GAP
        x = .CmdCreateTree.Left + _
            .CmdCreateTree.Width + GAP
        For i = (NumNodes - 1) / 2 To NumNodes - 1
            Set Nodes(i) = .NodeLabel(i)
            .NodeLabel(i).Caption = Format$(i)
            .NodeLabel(i).Move x, y
            x = x + wid + GAP
        Next i
    
        For i = (NumNodes - 3) / 2 To 0 Step -1
            Set Nodes(i) = .NodeLabel(i)
            .NodeLabel(i).Caption = Format$(i)
            x = (Nodes(2 * i + 1).Left + Nodes(2 * i + 2).Left) / 2
            y = Nodes(2 * i + 1).Top - hgt - GAP
            .NodeLabel(i).Move x, y
        Next i
    End With
End Sub

' ************************************************
' Draw lines connecting the centers of each node
' with its children.
' ************************************************
Public Sub DrawLines()
Dim node As Integer
Dim x1 As Single
Dim y1 As Single
Dim x2 As Single
Dim y2 As Single

    ' Quit if there's nothing to draw.
    If NumNodes < 1 Then Exit Sub

    With TheForm
        .Cls
        For node = 0 To NumNodes \ 2 - 1
            x1 = .NodeLabel(node).Left + .NodeLabel(node).Width / 2
            y1 = .NodeLabel(node).Top + .NodeLabel(node).Height / 2
            
            x2 = .NodeLabel(2 * node + 1).Left + .NodeLabel(2 * node + 1).Width / 2
            y2 = .NodeLabel(2 * node + 1).Top + .NodeLabel(2 * node + 1).Height / 2
            TheForm.Line (x1, y1)-(x2, y2)
            
            x2 = .NodeLabel(2 * node + 2).Left + .NodeLabel(2 * node + 2).Width / 2
            y2 = .NodeLabel(2 * node + 2).Top + .NodeLabel(2 * node + 2).Height / 2
            TheForm.Line (x1, y1)-(x2, y2)
        Next node
    End With
End Sub
' ************************************************
' Create a new preorder iterator.
' ************************************************
Public Function NewPreorderIterator() As Object
Dim iter As New PreorderIterator

    Set iter.TheTree = Me
    Set NewPreorderIterator = iter
End Function
' ************************************************
' Create a new postorder iterator.
' ************************************************
Public Function NewPostorderIterator() As Object
Dim iter As New PostorderIterator

    Set iter.TheTree = Me
    Set NewPostorderIterator = iter
End Function
' ************************************************
' Create a new inorder iterator.
' ************************************************
Public Function NewInorderIterator() As Object
Dim iter As New InorderIterator

    Set iter.TheTree = Me
    Set NewInorderIterator = iter
End Function
' ************************************************
' Create a new inorder iterator.
' ************************************************
Public Function NewBreadthFirstIterator() As Object
Dim iter As New BreadthFirstIterator

    Set iter.TheTree = Me
    Set NewBreadthFirstIterator = iter
End Function
' ************************************************
' Return a node's caption.
' ************************************************
Property Get NodeCaption(Index As Integer) As String
    NodeCaption = Nodes(Index).Caption
End Property

' ************************************************
' Return True if the node is in the tree.
' ************************************************
Public Function InTree(Index As Integer) As Boolean
    InTree = (Index < NumNodes)
End Function

