VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ShapePicture"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' ShapePic.CLS
'
' Picture class for serialization example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' A collection of command objects.
Private CmdObjects As Collection
Private LastCmd As Integer ' Index of last command.
' ************************************************
' Redo a command.
' ************************************************
Public Sub Redo()
    If LastCmd < CmdObjects.Count Then _
        LastCmd = LastCmd + 1
End Sub

' ************************************************
' Initialize the picture using a serialization.
' ************************************************
Property Let Serialization(txt As String)
Dim pos As Integer
Dim token_name As String
Dim token_value As String
Dim cmd As Object

    ' Start a new picture.
    NewPicture

    ' Read values until there are no more.
    GetSerialization txt, pos, token_name, token_value
    Do While token_name <> ""
        ' Make the object and make it unserialize itself.
        Set cmd = CommandFactory(token_name)
        If Not (cmd Is Nothing) Then _
            cmd.Serialization = token_value
        GetSerialization txt, pos, token_name, token_value
    Loop
    LastCmd = CmdObjects.Count
End Property
' ************************************************
' Return True if we can undo a command.
' ************************************************
Property Get CanUndo() As Boolean
    CanUndo = (LastCmd > 0)
End Property
' ************************************************
' Return True if we can redo a command.
' ************************************************
Property Get CanRedo() As Boolean
    CanRedo = (LastCmd < CmdObjects.Count)
End Property
' ************************************************
' Undo a command.
' ************************************************
Public Sub Undo()
    If LastCmd > 0 Then LastCmd = LastCmd - 1
End Sub

' ************************************************
' Start a fresh picture.
' ************************************************
Public Sub NewPicture()
    ' Start with an empty command list.
    Set CmdObjects = New Collection
    LastCmd = 0
End Sub

' ************************************************
' Serialize the picture. The routine could save
' all command objects and LastCmd. That would
' allow the user to redo commands that had been
' undone even after reloading the file! That would
' probably be unexpected, however, so we do not
' do it here.
' ************************************************
Property Get Serialization() As String
Dim txt As String
Dim i As Integer

    For i = 1 To LastCmd
        txt = txt & _
            TypeName(CmdObjects(i)) & "(" & _
            CmdObjects(i).Serialization & ")"
    Next i
    Serialization = txt
End Property

' ************************************************
' Create a new shape command object.
' ************************************************
Public Function CommandFactory(obj_name As String) As Object
Dim cmd As Object

    Select Case obj_name
        Case "EllipseCmd"
            Set cmd = New EllipseCmd
        Case "LineCmd"
            Set cmd = New LineCmd
        Case "RectangleCmd"
            Set cmd = New RectangleCmd
        Case Else
            Exit Function
    End Select

    ' If we are in the command history, remove any
    ' future commands.
    Do While CmdObjects.Count > LastCmd
        CmdObjects.Remove CmdObjects.Count
    Loop

    ' Add the command to the list of commands.
    CmdObjects.Add cmd
    LastCmd = CmdObjects.Count
    
    Set CommandFactory = cmd
End Function
' ************************************************
' Draw the objects on the picture box.
' ************************************************
Public Sub Draw(Canvas As PictureBox)
Dim i As Integer

    Canvas.Cls
    For i = 1 To LastCmd
        CmdObjects(i).Execute Canvas
    Next i
End Sub

