VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GarbageList"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' Garbage.CLS
'
' Unordered list with garbage collection.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' Normally WANT_FREE_PERCENT and MIN_FREE would
' have values more like .1 and 10. They are set
' to .5 and 1 to make it easier to see how the
' algorithm works.
Const WANT_FREE_PERCENT = 0.5   ' Try for 50% free space.
Const MIN_FREE = 1              ' Min unused space when resizing.

Private m_List() As Variant     ' The list array.
Private m_NumItems As Long      ' Last index in use.
Private m_ArraySize As Long     ' Size of the list array.
Private m_ShrinkWhen As Long    ' Shrink if m_NumItems < this.
Private m_GarbageCount As Long  ' The number of garbage entries.
Private m_MaxGarbage As Long    ' Collect when m_GarbageCount > this.

' ************************************************
' Collect the garbage.
' ************************************************
Private Sub CollectGarbage()
Dim i As Long
Dim good As Long

    ' Comment out for real applications.
    MsgBox "Collecting garbage."

    good = 1 ' The first good item goes here.
    For i = 1 To m_NumItems
        ' If it's not garbage,
        ' move it to its new location.
        If Not IsNull(m_List(i)) Then
            m_List(good) = m_List(i)
            good = good + 1
        End If
    Next i
    
    ' This is where the last good item is.
    m_NumItems = good - 1

    ' See if we should resize the list.
    If m_NumItems < m_ShrinkWhen Then ResizeList
End Sub

' ***********************************************
' Return an item from the list by position.
' ***********************************************
Function Item(ByVal position As Long) As Variant
    If position < 1 Or position > m_NumItems Then
        ' Out of bounds. Return Null.
        Item = Null
    Else
        ' Return the item.
        Item = m_List(position)
    End If
End Function
' ***********************************************
' Return the number of items.
' ***********************************************
Property Get NumItems() As Long
    NumItems = m_NumItems
End Property
' ***********************************************
' Return the size of the array.
' ***********************************************
Property Get ArraySize() As Long
    ArraySize = m_ArraySize
End Property
' ***********************************************
' Remove an item from the list.
' ***********************************************
Public Sub Remove(ByVal position As Long)
    m_List(position) = Null
    m_GarbageCount = m_GarbageCount + 1
    If m_GarbageCount > m_MaxGarbage Then CollectGarbage
End Sub
' ***********************************************
' Add an item.
' ***********************************************
Public Sub Add(ByVal value As Variant)
    m_NumItems = m_NumItems + 1
    If m_NumItems > m_ArraySize Then ResizeList
    m_List(m_NumItems) = value
End Sub

' ***********************************************
' Resize the list to have WANT_FREE_PERCENT
' unused entries. Set m_ShrinkWhen so we know to
' resize the array when the number of used
' entries is too small. Set this value to resize
' the array when there is more than twice
' the desired amount of memory free.
' ***********************************************
Private Sub ResizeList()
Dim want_free As Integer

    want_free = WANT_FREE_PERCENT * m_NumItems
    If want_free < MIN_FREE Then want_free = MIN_FREE
    m_ArraySize = m_NumItems + want_free
    ReDim Preserve m_List(1 To m_ArraySize)

    ' We will shrink the array if SL_NumItems < SL_ShrinkWhen.
    m_ShrinkWhen = m_NumItems - want_free
    
    ' Collect garbage when there are more than
    ' want_free garbage items in the list.
    m_MaxGarbage = want_free
    
    ' Comment when not testing.
    MsgBox "Resizing." & vbCrLf & vbCrLf & _
        "Items:" & Str$(m_ArraySize) & vbCrLf & _
        "Unused: " & Str$(want_free) & vbCrLf & _
        "ShrinkWhen:" & Str$(m_ShrinkWhen) & vbCrLf & _
        "MaxGarbage:" & Str$(m_MaxGarbage), _
        vbOKOnly, "SimpleList"
End Sub

