VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LinkedList"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' LnkList3.CLS
'
' Code for linked lists.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private m_NumItems As Long
Private m_Sentinel As New ListCell

Private m_CurrentCell As ListCell
' ************************************************
' Add an item after the indicated item.
' ************************************************
Public Sub AddAfter(ByVal after_me As Long, ByVal Value As Variant)
Dim ptr As ListCell
Dim new_cell As New ListCell

    If after_me > m_NumItems Or after_me < 0 _
        Then Exit Sub
    
    ' Find the insertion position.
    Set ptr = m_Sentinel
    Do While after_me > 0
        after_me = after_me - 1
        Set ptr = ptr.NextCell
    Loop
    
    ' Insert the item.
    new_cell.Value = Value
    Set new_cell.NextCell = ptr.NextCell
    Set ptr.NextCell = new_cell
    m_NumItems = m_NumItems + 1
End Sub
' ************************************************
' Return the current item's value.
' ************************************************
Public Function CurrentItem() As Variant
    If m_CurrentCell Is Nothing Then
        ' There's no current cell. Return Null.
        CurrentItem = Null
    Else
        CurrentItem = m_CurrentCell.Value
    End If
End Function
' ************************************************
' Move the current item to the next cell.
' ************************************************
Public Sub MoveNext()
    ' If there's no current cell, do nothing.
    If Not (m_CurrentCell Is Nothing) Then _
        Set m_CurrentCell = m_CurrentCell.NextCell
End Sub
' ************************************************
' Move the current item to the first cell.
' ************************************************
Public Sub MoveFirst()
    Set m_CurrentCell = m_Sentinel.NextCell
End Sub
' ************************************************
' Return true if the current item is beyond the
' end of the list.
' ************************************************
Public Function EndOfList() As Boolean
    EndOfList = (m_CurrentCell Is Nothing)
End Function

' ***********************************************
' Return an item from the list by position.
' ***********************************************
Function Item(ByVal position As Long) As Variant
Dim ptr As ListCell
    
    If position < 1 Or position > m_NumItems Then
        ' Out of bounds. Return Null.
        Item = Null
        Exit Function
    End If

    ' Find the item.
    Set ptr = m_Sentinel
    Do While position > 0
        position = position - 1
        Set ptr = ptr.NextCell
    Loop
    
    Item = ptr.Value
End Function
' ***********************************************
' Return the number of items.
' ***********************************************
Property Get NumItems() As Long
    NumItems = m_NumItems
End Property
' ************************************************
' Remove the item after the indicated item.
' ************************************************
Public Sub RemoveAfter(ByVal after_me As Long)
Dim ptr As ListCell

    If after_me >= m_NumItems Or after_me < 0 _
        Then Exit Sub
    
    ' Find the deletion position.
    Set ptr = m_Sentinel
    Do While after_me > 0
        after_me = after_me - 1
        Set ptr = ptr.NextCell
    Loop
    
    ' If we are deleting the current cell, set
    ' m_CurrentCell to the previous cell.
    If ptr.NextCell Is m_CurrentCell Then _
        Set m_CurrentCell = ptr
    
    ' Skip the following item.
    Set ptr.NextCell = ptr.NextCell.NextCell
    m_NumItems = m_NumItems - 1
End Sub
' ************************************************
' Clear the list.
' ************************************************
Public Sub ClearList()
    Set m_Sentinel.NextCell = Nothing
    Set m_CurrentCell = Nothing
    m_NumItems = 0
End Sub

