VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ThreadedList"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' ThList.CLS
'
' Code for threaded linked lists.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private m_TopSentinel As New ThreadedCell
Private m_BottomSentinel As New ThreadedCell

Private m_CurrentCell As ThreadedCell
' ************************************************
' Add an item to the list.
' ************************************************
Public Sub AddItem(LastName As String, FirstName As String, SSN As String, Sex As String, JobClass As Integer)
Dim ptr As ThreadedCell
Dim nxt As ThreadedCell
Dim new_cell As New ThreadedCell
Dim new_name As String
Dim next_name As String

    ' Save the new cell's values.
    With new_cell
        .LastName = LastName
        .FirstName = FirstName
        .SSN = SSN
        .Sex = Sex
        .JobClass = JobClass
    End With
    
    ' ********************
    ' Add in Name threads.
    ' ********************
    ' See where the new cell belongs in the
    ' NextName thread.
    new_name = LastName & ", " & FirstName
    Set ptr = m_TopSentinel
    Do
        Set nxt = ptr.NextName
        next_name = nxt.LastName & ", " & nxt.FirstName
        If next_name >= new_name Then Exit Do
        Set ptr = nxt
    Loop
    
    ' Insert the new cell.
    Set new_cell.NextName = nxt
    Set new_cell.PrevName = ptr
    Set ptr.NextName = new_cell
    Set nxt.PrevName = new_cell
    
    ' ******************
    ' Add in SSN thread.
    ' ******************
    Set ptr = m_TopSentinel
    Do
        Set nxt = ptr.NextSSN
        If nxt.SSN >= SSN Then Exit Do
        Set ptr = nxt
    Loop
    
    ' Insert the item.
    Set new_cell.NextSSN = nxt
    Set ptr.NextSSN = new_cell
    
    ' *************************
    ' Add in job class threads.
    ' *************************
    Set ptr = m_TopSentinel
    Do
        Set nxt = ptr.NextJobClass
        If nxt.JobClass >= JobClass Then Exit Do
        Set ptr = nxt
    Loop
    
    ' Insert the item.
    Set new_cell.NextJobClass = nxt
    Set new_cell.PrevJobClass = ptr
    Set ptr.NextJobClass = new_cell
    Set nxt.PrevJobClass = new_cell
End Sub
' ************************************************
' Return the current cell.
' ************************************************
Public Function CurrentItem() As ThreadedCell
    Set CurrentItem = m_CurrentCell
End Function
' ************************************************
' Move the current item to the next cell in a
' thread.
' ************************************************
Public Sub MoveNext(thread As Integer)
    ' If there's no current cell, do nothing.
    If m_CurrentCell Is Nothing Then Exit Sub
    
    Select Case thread
        Case 0  ' NextName
            Set m_CurrentCell = m_CurrentCell.NextName
        Case 1  ' PrevName
            Set m_CurrentCell = m_CurrentCell.PrevName
        Case 2  ' NextSSN
            Set m_CurrentCell = m_CurrentCell.NextSSN
        Case 3  ' NextJobClass
            Set m_CurrentCell = m_CurrentCell.NextJobClass
        Case 4  ' PrevJobClass
            Set m_CurrentCell = m_CurrentCell.PrevJobClass
    End Select
End Sub

' ************************************************
' Move the current item to the first cell in a
' thread.
' ************************************************
Public Sub MoveFirst(thread As Integer)
    Select Case thread
        Case 0  ' NextName
            Set m_CurrentCell = m_TopSentinel.NextName
        Case 1  ' PrevName
            Set m_CurrentCell = m_BottomSentinel.PrevName
        Case 2  ' NextSSN
            Set m_CurrentCell = m_TopSentinel.NextSSN
        Case 3  ' NextJobClass
            Set m_CurrentCell = m_TopSentinel.NextJobClass
        Case 4  ' PrevJobClass
            Set m_CurrentCell = m_BottomSentinel.PrevJobClass
    End Select
End Sub
' ************************************************
' Return true if the current item is beyond the
' end of the list.
' ************************************************
Public Function EndOfList() As Boolean
    EndOfList = _
        ((m_CurrentCell Is m_TopSentinel) Or _
         (m_CurrentCell Is m_BottomSentinel))
End Function

' ************************************************
' Initialize the sentinels.
' ************************************************
Private Sub Class_Initialize()
    Set m_TopSentinel.NextName = m_BottomSentinel
    Set m_TopSentinel.NextSSN = m_BottomSentinel
    Set m_TopSentinel.NextJobClass = m_BottomSentinel
    Set m_BottomSentinel.PrevName = m_TopSentinel
    Set m_BottomSentinel.PrevJobClass = m_TopSentinel
    
    Set m_CurrentCell = m_TopSentinel
    
    ' Give m_TopSentinel values that come before
    ' all others.
    With m_TopSentinel
        .LastName = ""
        .FirstName = ""
        .SSN = ""
        .JobClass = -32768
        .Sex = ""
    End With
    
    ' Give m_BottomSentinel values that come after
    ' all others.
    With m_BottomSentinel
        .LastName = "~"
        .FirstName = "~"
        .SSN = "~"
        .JobClass = 32767
        .Sex = "~"
    End With
End Sub

' ************************************************
' Remove circular references so the list is
' properly destroyed.
' ************************************************
Private Sub Class_Terminate()
Dim ptr As ThreadedCell

    ' Remove all pointers except NextName in all
    ' cells including m_BottomSentinel. To do
    ' this, ptr must traverse the entire list but
    ' it does not matter what order we use.
    Set ptr = m_TopSentinel.NextName
    Do While Not (ptr Is Nothing)
        Set ptr.PrevName = Nothing
        Set ptr.NextSSN = Nothing
        Set ptr.NextJobClass = Nothing
        Set ptr.PrevJobClass = Nothing
        Set ptr = ptr.NextName
    Loop
End Sub

