Attribute VB_Name = "ArrayQStuff"
' ************************************************
' ArrayQ.BAS
'
' Code for array-based queue.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' Normally WANT_FREE_PERCENT and MIN_FREE would
' have values more like .1 and 10. They are set
' to .5 and 2 to make it easier to see how the
' algorithm works.
Const WANT_FREE_PERCENT = 0.5   ' Try for 50% free space.
Const MIN_FREE = 2              ' Min unused space when resizing.

Global AQ_Queue() As Variant    ' The queue array.
Global AQ_Max As Integer        ' Largest index in the array.
Global AQ_Front As Integer      ' The next item to be removed.
Global AQ_Back As Integer       ' Where the next new item goes.
Global AQ_ResizeWhen As Integer ' Resize if AQ_Front >= this.

' ************************************************
' Add an item to the queue.
' ************************************************
Sub AQ_EnterQueue(value As Variant)
    If AQ_Back > AQ_Max Then AQ_ResizeQueue
    AQ_Queue(AQ_Back) = value
    AQ_Back = AQ_Back + 1
End Sub

' ************************************************
' Remove the top item from the queue.
' ************************************************
Sub AQ_LeaveQueue(value As Variant)
    value = AQ_Queue(AQ_Front)
    AQ_Front = AQ_Front + 1
    If AQ_Front > AQ_ResizeWhen Then AQ_ResizeQueue
End Sub

' ************************************************
' There are two reasons to resize the queue:
'   1. We have reached the end and need more room
'   2. We have too much empty space at the beginning
'
' In either case we move all of the items in the array
' to the front of the array and then we resize so we have
' the correct amount of free space available.
'
' AQ_ResizeWhen is set so we resize when
' AQ_Front > AQ_ResizeWhen.
' ************************************************
Sub AQ_ResizeQueue()
Dim want_free As Integer
Dim i As Integer

    ' Move the entries to the beginning of the array.
    For i = AQ_Front To AQ_Back - 1
        AQ_Queue(i - AQ_Front) = AQ_Queue(i)
    Next i
    AQ_Back = AQ_Back - AQ_Front
    AQ_Front = 0

    ' Resize the array.
    want_free = WANT_FREE_PERCENT * (AQ_Back - AQ_Front)
    If want_free < MIN_FREE Then want_free = MIN_FREE
    AQ_Max = AQ_Back + want_free - 1
    ReDim Preserve AQ_Queue(0 To AQ_Max)

    ' We will resize the array when AQ_Front > AQ_ResizeWhen.
    AQ_ResizeWhen = want_free

    MsgBox "Resizing array queue." & vbCrLf & vbCrLf & _
        "Items:" & Str$(AQ_Max + 1) & vbCrLf & _
        "Unused: " & Str$(want_free) & vbCrLf & _
        "ResizeWhen:" & Str$(AQ_ResizeWhen)
End Sub

