VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ArrayQueue"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' ArrayQ.CLS
'
' Code for array-based queue class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' Normally WANT_FREE_PERCENT and MIN_FREE would
' have values more like .1 and 10. They are set
' to .5 and 2 to make it easier to see how the
' algorithm works.
Private Const WANT_FREE_PERCENT = 0.5   ' Try for 50% free space.
Private Const MIN_FREE = 2              ' Min unused space when resizing.

Private Queue() As Variant    ' The queue array.
Private Max As Integer        ' Largest index in the array.
Private Front As Integer      ' The next item to be removed.
Private Back As Integer       ' Where the next new item goes.
Private ResizeWhen As Integer ' Resize if AQ_Front >= this.
' ************************************************
' Remove the top item from the queue.
' ************************************************
Property Get LeaveQueue() As Variant
    If Front = Back Then
        Set LeaveQueue = Nothing
    Else
        LeaveQueue = Queue(Front)
        Front = Front + 1
        If Front > ResizeWhen Then ResizeQueue
    End If
End Property
' ************************************************
' Return true if the queue is empty.
' ************************************************
Property Get QueueEmpty() As Boolean
    QueueEmpty = (Front = Back)
End Property
' ************************************************
' There are two reasons to resize the queue:
'   1. We have reached the end and need more room
'   2. We have too much empty space at the beginning
'
' In either case we move all of the items in the
' array to the front of the array and then we
' resize so we have the correct amount of free
' space available.
'
' ResizeWhen is set so we resize when
' Front > ResizeWhen.
' ************************************************
Private Sub ResizeQueue()
Dim want_free As Integer
Dim i As Integer

    ' Move the entries to the beginning of the array.
    For i = Front To Back - 1
        Queue(i - Front) = Queue(i)
    Next i
    Back = Back - Front
    Front = 0

    ' Resize the array.
    want_free = WANT_FREE_PERCENT * (Back - Front)
    If want_free < MIN_FREE Then want_free = MIN_FREE
    Max = Back + want_free - 1
    ReDim Preserve Queue(0 To Max)

    ' We will resize the array when
    ' Front > ResizeWhen.
    ResizeWhen = want_free

    MsgBox "Resizing array queue." & vbCrLf & vbCrLf & _
        "Items:" & Str$(Max + 1) & vbCrLf & _
        "Unused: " & Str$(want_free) & vbCrLf & _
        "ResizeWhen:" & Str$(ResizeWhen)
End Sub
' ************************************************
' Add an item to the queue.
' ************************************************
Public Sub EnterQueue(Value As Variant)
    If Back > Max Then ResizeQueue
    Queue(Back) = Value
    Back = Back + 1
End Sub

' ************************************************
' Return a textual representation of the queue.
' ************************************************
Property Get TextRepresentation() As String
Dim txt As String
Dim i As Integer
    
    ' Unused items before the real items.
    For i = 0 To Front - 1
        txt = txt & "<unused>" & vbCrLf
    Next i

    ' The real items.
    For i = Front To Back - 1
        txt = txt & Queue(i) & vbCrLf
    Next i

    ' Display unused items after the real items.
    For i = Back To Max
        txt = txt & "<unused>" & vbCrLf
    Next i

    TextRepresentation = txt
End Property


' ************************************************
' This makes us initialize the queue the first
' time we use it.
' ************************************************
Private Sub Class_Initialize()
    Max = -1
End Sub


