Attribute VB_Name = "CircleQStuff"
' ************************************************
' CircleQ.BAS
'
' Code for circular queue example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' Normally WANT_FREE_PERCENT and MIN_FREE would
' have values more like .1 and 10. They are set to
' .5 and 2 to make it easier to see how the
' algorithm works.
Const WANT_FREE_PERCENT = 0.5   ' Try for 50% free space.
Const MIN_FREE = 2              ' Min unused space when resizing.

Global CQ_Queue() As Variant    ' The queue array.
Global CQ_Size As Integer       ' # entries in array.
Global CQ_Front As Integer      ' The next item to be removed.
Global CQ_Back As Integer       ' Where the next new item goes.
Global CQ_NumInQueue As Integer ' # items in the queue
Global CQ_ShrinkWhen As Integer ' Shrink if CQ_NumInQueue < this.

' ************************************************
' Add an item to the queue.
' ************************************************
Sub CQ_EnterQueue(value As Variant)
    If CQ_NumInQueue >= CQ_Size Then CQ_ResizeQueue
    
    CQ_Queue(CQ_Back) = value
    CQ_Back = (CQ_Back + 1) Mod CQ_Size
    CQ_NumInQueue = CQ_NumInQueue + 1
End Sub

' ************************************************
' Remove an item from the queue.
' ************************************************
Sub CQ_LeaveQueue(value As Variant)
    If CQ_NumInQueue <= 0 Then Exit Sub

    value = CQ_Queue(CQ_Front)
    
    ' This is not really necessary but makes it easier
    ' to display the items not in use.
    CQ_Queue(CQ_Front) = "<unused>"
    
    CQ_Front = (CQ_Front + 1) Mod CQ_Size
    
    CQ_NumInQueue = CQ_NumInQueue - 1
    If CQ_NumInQueue < CQ_ShrinkWhen Then CQ_ResizeQueue
End Sub

' ************************************************
' Resize the list to have WANT_FREE_PERCENT unused space.
' Set CQ_ShrinkWhen so we know to resize the array when
' the number of entries is less than CQ_ShrinkWhen. Set
' this value to resize the array when there is more than
' twice the desired amount of memory free.
' ************************************************
Sub CQ_ResizeQueue()
Dim temp() As String
Dim want_free As Integer
Dim i As Integer
Dim tmp As String
Dim num_tmp As Integer

    ' Copy items into temporary array.
    num_tmp = CQ_NumInQueue - 1
    If num_tmp < 0 Then num_tmp = 0
    ReDim temp(0 To num_tmp)
    For i = 0 To CQ_NumInQueue - 1
        temp(i) = CQ_Queue((i + CQ_Front) Mod CQ_Size)
    Next i
    
    ' Resize the array
    want_free = WANT_FREE_PERCENT * CQ_NumInQueue
    If want_free < MIN_FREE Then want_free = MIN_FREE
    CQ_Size = CQ_NumInQueue + want_free
    ReDim CQ_Queue(0 To CQ_Size - 1)
    
    ' Copy items back out of the temporary array.
    For i = 0 To CQ_NumInQueue - 1
        CQ_Queue(i) = temp(i)
    Next i
    CQ_Front = 0
    CQ_Back = CQ_NumInQueue

    ' This is not really necessary but makes it easier
    ' to display the items not in use.
    For i = CQ_NumInQueue To CQ_Size - 1
        CQ_Queue(i) = "<unused>"
    Next i

    ' We will shrink the array when CQ_NumInQueue < CQ_ShrinkWhen.
    CQ_ShrinkWhen = CQ_Size - 2 * want_free
    
    ' Don't resize tiny queues. This can get us in trouble
    ' with "ReDim temp(0 To CQ_NumInQueue - 1)" above and
    ' just plain looks silly!
    If CQ_ShrinkWhen < 3 Then CQ_ShrinkWhen = 0

    MsgBox "Resizing circular queue." & _
        vbCrLf & vbCrLf & _
        "QueueSize:" & Str$(CQ_Size) & vbCrLf & _
        "Unused: " & Str$(want_free) & vbCrLf & _
        "ShrinkWhen:" & Str$(CQ_ShrinkWhen)
End Sub

