VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CircleQueue"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' CircleQ.CLS
'
' Circular queue class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' Normally WANT_FREE_PERCENT and MIN_FREE would
' have values more like .1 and 10. They are set
' to .5 and 2 to make it easier to see how the
' algorithm works.
Private Const WANT_FREE_PERCENT = 0.5   ' Try for 50% free space.
Private Const MIN_FREE = 2              ' Min unused space when resizing.

Private Queue() As Variant    ' The queue array.
Private Size As Integer       ' # entries in array.
Private Front As Integer      ' The next item to be removed.
Private Back As Integer       ' Where the next new item goes.
Private NumInQueue As Integer ' # items in the queue
Private ShrinkWhen As Integer ' Shrink if NumInQueue < this.

' ************************************************
' Return the index of the front of the queue.
' This is necessary only for the fancy display
' provided by program CircleQ2.
' ************************************************
Property Get FrontIndex() As Integer
    FrontIndex = Front
End Property
' ************************************************
' Return the index of the back of the queue.
' This is necessary only for the fancy display
' provided by program CircleQ2.
' ************************************************
Property Get BackIndex() As Integer
    BackIndex = Back
End Property
' ************************************************
' Return true if the queue is empty.
' ************************************************
Property Get NumItems() As Integer
    NumItems = NumInQueue
End Property
' ************************************************
' Return a queue item by index. Note that this
' procedure does not check that the item is a
' real item and not an empty position.
' ************************************************
Property Get Item(Index As Integer) As String
    Item = Queue(Index)
End Property
' ************************************************
' Return the number of entries in the queue array.
' This is necessary only for the fancy display
' provided by program CircleQ2.
' ************************************************
Property Get QueueSize() As Integer
    QueueSize = Size
End Property
' ************************************************
' Resize the list to have WANT_FREE_PERCENT unused
' space. Set ShrinkWhen so we know to resize the
' array when the number of entries is less than
' ShrinkWhen. Set this value to resize the array
' when there is more than twice the desired amount
' of memory free.
' ************************************************
Private Sub ResizeQueue()
Dim temp() As String
Dim want_free As Integer
Dim i As Integer
Dim tmp As String
Dim num_tmp As Integer

    ' Copy items into temporary array.
    num_tmp = NumInQueue - 1
    If num_tmp < 0 Then num_tmp = 0
    ReDim temp(0 To num_tmp)
    For i = 0 To NumInQueue - 1
        temp(i) = Queue((i + Front) Mod Size)
    Next i

    ' At this point the entries are in positions
    ' temp(0) through temp(NumInQueue).
    
    ' Resize the array
    want_free = WANT_FREE_PERCENT * NumInQueue
    If want_free < MIN_FREE Then want_free = MIN_FREE
    Size = NumInQueue + want_free
    ReDim Queue(0 To Size - 1)
    
    ' Copy items back out of the temporary array.
    For i = 0 To NumInQueue - 1
        Queue(i) = temp(i)
    Next i
    Front = 0
    Back = NumInQueue

    ' We will shrink the array when
    ' NumInQueue < ShrinkWhen.
    ShrinkWhen = Size - 2 * want_free
    
    ' Don't resize tiny queues. This can get us in
    ' trouble with "ReDim temp(0 To NumInQueue - 1)"
    ' above and just plain looks silly!
    If ShrinkWhen < 3 Then ShrinkWhen = 0

    MsgBox "Resizing circular queue." & _
        vbCrLf & vbCrLf & _
        "QueueSize:" & Str$(Size) & vbCrLf & _
        "Unused: " & Str$(want_free) & vbCrLf & _
        "ShrinkWhen:" & Str$(ShrinkWhen)
End Sub
' ************************************************
' Add an item to the queue.
' ************************************************
Public Sub EnterQueue(value As Variant)
    If NumInQueue >= Size Then ResizeQueue

    Queue(Back) = value
    Back = (Back + 1) Mod Size
    NumInQueue = NumInQueue + 1
End Sub
' ************************************************
' Remove the top item from the queue.
' ************************************************
Property Get LeaveQueue() As Variant
    If NumInQueue <= 0 Then
        Set LeaveQueue = Nothing
        Exit Property
    End If

    LeaveQueue = Queue(Front)
    Front = (Front + 1) Mod Size
    NumInQueue = NumInQueue - 1
    If NumInQueue < ShrinkWhen Then ResizeQueue
End Property
' ************************************************
' Return a textual representation of the queue.
' ************************************************
Property Get TextRepresentation() As String
Dim txt As String
Dim i As Integer
    
    For i = 0 To NumInQueue - 1
        txt = txt & Queue((i + Front) Mod Size) & vbCrLf
    Next i
    TextRepresentation = txt
End Property


