VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LinkedListQueue"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' LinkedQ.CLS
'
' Linked list based queue class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private TopSentinel As New LinkedQCell
Private BottomSentinel As New LinkedQCell

' ************************************************
' Add a new item to the queue.
' ************************************************
Public Sub EnterQueue(Value As Variant)
Dim new_cell As New LinkedQCell

    ' Add the new cell above BottomSentinel.
    new_cell.Value = Value
    Set new_cell.PrevCell = BottomSentinel.PrevCell
    Set new_cell.PrevCell.NextCell = new_cell
    Set new_cell.NextCell = BottomSentinel
    Set BottomSentinel.PrevCell = new_cell
End Sub
' ************************************************
' Remove an item from the queue.
' ************************************************
Property Get LeaveQueue() As Variant
Dim cell As LinkedQCell

    ' Remove the cell just after the TopSentinel.
    Set cell = TopSentinel.NextCell
    Set TopSentinel.NextCell = cell.NextCell
    Set cell.NextCell.PrevCell = TopSentinel
    
    ' We do not need to set cell's NextCell and
    ' PrevCell values to Nothing. Its reference
    ' count will be zero when this routine exits
    ' so it will be destroyed anyway.
    
    LeaveQueue = cell.Value
End Property
' ************************************************
' Return true if the queue is empty.
' ************************************************
Property Get QueueIsEmpty() As Boolean
    QueueIsEmpty = (TopSentinel.NextCell Is BottomSentinel)
End Property


' ************************************************
' Return a textual representation of the queue.
' ************************************************
Property Get TextRepresentation() As String
Dim cell As LinkedQCell
Dim txt As String
    
    Set cell = TopSentinel.NextCell
    Do While Not (cell Is BottomSentinel)
        txt = txt & cell.Value & vbCrLf
        Set cell = cell.NextCell
    Loop
    TextRepresentation = txt
End Property
' ************************************************
' Initialize the queue.
' ************************************************
Private Sub Class_Initialize()
    Set TopSentinel.NextCell = BottomSentinel
    Set BottomSentinel.PrevCell = TopSentinel
End Sub
