VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LinkedPriorityQueue"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' PriList.CLS
'
' Linked list based priority queue class.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Private TopSentinel As New PriorityQCell
Private BottomSentinel As New PriorityQCell

' ************************************************
' Add a new item to the queue.
' ************************************************
Public Sub EnterQueue(Value As Variant, Priority As Integer)
Dim new_cell As New PriorityQCell
Dim cell As PriorityQCell

    new_cell.Value = Value
    new_cell.Priority = Priority
    
    ' See where the item belongs.
    Set cell = TopSentinel
    Do While cell.NextCell.Priority > Priority
        Set cell = cell.NextCell
    Loop
    
    ' Add the new cell.
    Set new_cell.NextCell = cell.NextCell
    Set cell.NextCell = new_cell
End Sub
' ************************************************
' Remove an item from the queue.
' ************************************************
Public Sub LeaveQueue(Value As Variant, Priority As Integer)
Dim cell As PriorityQCell

    If QueueIsEmpty Then
        Set Value = Nothing
        Priority = -32768
        Exit Sub
    End If
    
    ' Remove the cell just after the TopSentinel.
    Set cell = TopSentinel.NextCell
    Set TopSentinel.NextCell = cell.NextCell
    
    Value = cell.Value
    Priority = cell.Priority
End Sub
' ************************************************
' Return true if the queue is empty.
' ************************************************
Property Get QueueIsEmpty() As Boolean
    QueueIsEmpty = (TopSentinel.NextCell Is BottomSentinel)
End Property


' ************************************************
' Return a textual representation of the queue.
' ************************************************
Property Get TextRepresentation() As String
Dim cell As PriorityQCell
Dim txt As String
    
    Set cell = TopSentinel.NextCell
    Do While Not (cell Is BottomSentinel)
        txt = txt & cell.Value & " (" & _
            Format$(cell.Priority) & ")" & vbCrLf
        Set cell = cell.NextCell
    Loop
    TextRepresentation = txt
End Property
' ************************************************
' Initialize the queue.
' ************************************************
Private Sub Class_Initialize()
    Set TopSentinel.NextCell = BottomSentinel
    BottomSentinel.Priority = -32768
End Sub
