VERSION 4.00
Begin VB.Form FStarForm 
   Appearance      =   0  'Flat
   Caption         =   "FStar"
   ClientHeight    =   3990
   ClientLeft      =   2220
   ClientTop       =   1920
   ClientWidth     =   4095
   BeginProperty Font {0BE35203-8F91-11CE-9DE3-00AA004BB851} 
      Name            =   "MS Sans Serif"
      Size            =   8.25
      Charset         =   0
      Weight          =   700
      Underline       =   0   'False
      Italic          =   0   'False
      Strikethrough   =   0   'False
   EndProperty
   ForeColor       =   &H80000008&
   Height          =   4680
   Icon            =   "FStar.frx":0000
   Left            =   2160
   LinkTopic       =   "Form1"
   ScaleHeight     =   3990
   ScaleWidth      =   4095
   Top             =   1290
   Width           =   4215
   Begin VB.CommandButton CmdRemove 
      Appearance      =   0  'Flat
      BackColor       =   &H80000005&
      Caption         =   "Remove"
      Enabled         =   0   'False
      Height          =   495
      Left            =   1440
      TabIndex        =   1
      Top             =   120
      Width           =   1215
   End
   Begin VB.CommandButton CmdAddChild 
      Appearance      =   0  'Flat
      BackColor       =   &H80000005&
      Caption         =   "Add Child"
      Enabled         =   0   'False
      Height          =   495
      Left            =   120
      TabIndex        =   0
      Top             =   120
      Width           =   1215
   End
   Begin VB.Label NodeLabel 
      Alignment       =   2  'Center
      BorderStyle     =   1  'Fixed Single
      Caption         =   "0"
      Height          =   255
      Index           =   0
      Left            =   120
      TabIndex        =   2
      Top             =   720
      Width           =   375
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileExit 
         Caption         =   "E&xit"
      End
   End
   Begin VB.Menu mnuHelp 
      Caption         =   "&Help"
      Begin VB.Menu mnuHelpAbout 
         Caption         =   "&About This Program..."
      End
   End
End
Attribute VB_Name = "FStarForm"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' FSTAR.FRM
'
' Main form for forward star N-ary tree example
' program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Const XGAP = 60
Const YGAP = 120

Dim Wid As Single
Dim Hgt As Single

' The currently selected node.
Dim SelectedNode As Integer

' Index of last NodeLabel control loaded.
Dim MaxBox As Integer
' ************************************************
' Add a new child to the selected node.
' ************************************************
Private Sub CmdAddChild_Click()
Dim child As Integer
Dim node As Integer
    
    node = NewNode()
    AddLink SelectedNode, node

    SelectNode -1
    DisplayTree
End Sub

' *******************************************************
' Rmove the selected node and its descendants.
' *******************************************************
Private Sub CmdRemove_Click()
Dim parent As Integer
Dim link As Integer

    If SelectedNode <= 0 Then Exit Sub  ' Shouldn't happen
    
    FindParent SelectedNode, parent, link
    If parent < 0 Then Exit Sub         ' Shouldn't happen

    FreeNodeAndChildren parent, link, SelectedNode

    SelectNode -1
    DisplayTree
End Sub

' ************************************************
' Display the node at the indicated y value. Start
' the descendants of the node at xmin. Return the
' maximum x position used by this node through the
' parameter xmin.
' ************************************************
Private Sub DisplayNode(node As Integer, xmin As Single, ByVal y As Single)
Dim link As Integer
Dim x1 As Single
Dim y1 As Single

    ' First display the children
    y1 = y + Hgt + YGAP
    x1 = xmin
    For link = FirstLink(node) To FirstLink(node + 1) - 1
        DisplayNode ToNode(link), x1, y1
        x1 = x1 + XGAP
    Next link
    
    ' Now display this node
    If x1 > xmin Then
        xmin = (xmin + x1 - XGAP - Wid) / 2
        x1 = x1 - XGAP
    Else
        x1 = x1 + Wid
    End If
    NodeLabel(node).Move xmin, y
    NodeLabel(node).Caption = NodeCaption(node)
    xmin = x1
End Sub
' ************************************************
' Display this tree recursively by displaying the
' root.
' ************************************************
Private Sub DisplayTree()
Dim xmin As Single

    xmin = CmdAddChild.Left
    DisplayNode 0, xmin, NodeLabel(0).Top
    Refresh
End Sub

' ************************************************
' Draw lines connecting the centers of each node
' and its children.
' ************************************************
Private Sub DrawLines(node As Integer)
Dim link As Integer
Dim x As Single
Dim y As Single
Dim to_node As Integer

    x = NodeLabel(node).Left + NodeLabel(node).Width / 2
    y = NodeLabel(node).Top + NodeLabel(node).Height / 2
    
    ' Draw lines to children
    For link = FirstLink(node) To FirstLink(node + 1) - 1
        to_node = ToNode(link)
        Line (x, y)-(NodeLabel(to_node).Left + NodeLabel(to_node).Width / 2, NodeLabel(to_node).Top + NodeLabel(to_node).Height / 2)
        DrawLines to_node
    Next link
End Sub

Private Sub Form_Load()
    Wid = NodeLabel(0).Width
    Hgt = NodeLabel(0).Height
    
    SelectedNode = -1
    InitTree
    DisplayTree
End Sub

' *******************************************************
' Draw lines between the nodes.
' *******************************************************
Private Sub Form_Paint()
    DrawLines 0
End Sub

' ************************************************
' Redirect control box unload to mnuExit.
' ************************************************
Private Sub Form_QueryUnload(Cancel As Integer, UnloadMode As Integer)
    Unload Me
End Sub

Private Sub mnuFileExit_Click()
    Unload Me
End Sub

' *******************************************************
' Present help about this program.
' *******************************************************
Private Sub mnuHelpAbout_Click()
Dim txt As String
    
    txt = "This program allows you to manipulate an N-ary tree using forward star." & vbCrLf & vbCrLf
    txt = txt & "Select a node in the tree and the appropriate buttons will become enabled." & vbCrLf & vbCrLf
    txt = txt & "If you remove a node, all of its descendants will be removed as well."

    MsgBox txt, vbOKOnly, "Help About ... FStar"
End Sub

' *******************************************************
' Select the node the user clicked on.
' *******************************************************
Private Sub NodeLabel_Click(Index As Integer)
    SelectNode Index
End Sub

' ************************************************
' Select this node.
' ************************************************
Private Sub SelectNode(node As Integer)
    If SelectedNode >= 0 And SelectedNode < NumNodes Then
        NodeLabel(SelectedNode).BackColor = BackColor
        NodeLabel(SelectedNode).ForeColor = ForeColor
    End If

    SelectedNode = node
    
    If SelectedNode < 0 Then
        CmdRemove.Enabled = False
        CmdAddChild.Enabled = False
    Else
        NodeLabel(SelectedNode).BackColor = ForeColor
        NodeLabel(SelectedNode).ForeColor = BackColor
        CmdRemove.Enabled = (SelectedNode <> 0)
        CmdAddChild.Enabled = True
    End If
End Sub

