VERSION 4.00
Begin VB.Form QtreeForm 
   Appearance      =   0  'Flat
   BackColor       =   &H00000000&
   Caption         =   "Qtree"
   ClientHeight    =   5475
   ClientLeft      =   1725
   ClientTop       =   1005
   ClientWidth     =   5475
   FillStyle       =   0  'Solid
   BeginProperty Font {0BE35203-8F91-11CE-9DE3-00AA004BB851} 
      Name            =   "MS Sans Serif"
      Size            =   8.25
      Charset         =   0
      Weight          =   700
      Underline       =   0   'False
      Italic          =   0   'False
      Strikethrough   =   0   'False
   EndProperty
   ForeColor       =   &H000000FF&
   Height          =   6165
   Icon            =   "Qtree.frx":0000
   Left            =   1665
   LinkTopic       =   "Form1"
   ScaleHeight     =   5475
   ScaleWidth      =   5475
   Top             =   375
   Width           =   5595
   Begin VB.Label CompLabel 
      BorderStyle     =   1  'Fixed Single
      Height          =   255
      Left            =   960
      TabIndex        =   1
      Top             =   30
      Width           =   615
   End
   Begin VB.Label Label1 
      Appearance      =   0  'Flat
      BackColor       =   &H00000000&
      Caption         =   "Examined"
      ForeColor       =   &H00FFFFFF&
      Height          =   255
      Left            =   90
      TabIndex        =   0
      Top             =   30
      Width           =   975
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileExit 
         Caption         =   "E&xit"
      End
   End
   Begin VB.Menu mnuOpt 
      Caption         =   "&Options"
      Begin VB.Menu mnuOptUseQtree 
         Caption         =   "&Use Quadtree"
         Checked         =   -1  'True
      End
   End
   Begin VB.Menu mnuHelp 
      Caption         =   "&Help"
      Begin VB.Menu mnuHelpAbout 
         Caption         =   "&About This Program..."
      End
   End
End
Attribute VB_Name = "QtreeForm"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' Qtree.FRM
'
' Main form for quadtree example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Dim Highlighted As QtreeItem

Dim AllItems As New Collection
Dim Root As New QtreeNode

' Drawing area bounds.
Dim Gxmin As Single
Dim Gxmax As Single
Dim Gymin As Single
Dim Gymax As Single

' ************************************************
' If this quadtree node has children, draw
' dividers between the children.
' ************************************************
Private Sub DisplayQtree(node As QtreeNode, xmin As Single, xmax As Single, ymin As Single, ymax As Single)
Dim xmid As Single
Dim ymid As Single
    
    ' If it's a leaf node, do nothing.
    If node.SWchild Is Nothing Then Exit Sub
    
    ' Draw the dividers.
    xmid = (xmin + xmax) / 2
    ymid = (ymin + ymax) / 2
    Line (xmin, ymid)-(xmax, ymid)
    Line (xmid, ymin)-(xmid, ymax)

    ' Recursively display the children.
    DisplayQtree node.NWchild, xmin, xmid, ymin, ymid
    DisplayQtree node.NEchild, xmid, xmax, ymin, ymid
    DisplayQtree node.SWchild, xmin, xmid, ymid, ymax
    DisplayQtree node.SEchild, xmid, xmax, ymid, ymax
End Sub

' ************************************************
' Draw a dot for each item.
' ************************************************
Private Sub DrawDots()
Dim new_item As QtreeItem

    For Each new_item In AllItems
        PSet (new_item.X, new_item.Y), vbRed
    Next new_item
End Sub

' ************************************************
' If the Use Quadtree option is checked, display
' the quadtree.
' ************************************************
Private Sub DrawQtree()
    If Not mnuOptUseQtree.Checked Then Exit Sub

    DisplayQtree Root, Gxmin, Gxmax, Gymin, Gymax
End Sub

' ************************************************
' Locate the point the slow way.
' ************************************************
Private Function FindWithoutQtree(X As Single, Y As Single) As QtreeItem
Dim best_dist As Single
Dim new_item As QtreeItem
Dim new_dist As Single
Dim Dx As Single
Dim Dy As Single

    best_dist = 1000000000
    Set FindWithoutQtree = Nothing
    
    For Each new_item In AllItems
        Dx = new_item.X - X
        Dy = new_item.Y - Y
        new_dist = Dx * Dx + Dy * Dy
        If best_dist > new_dist Then
            best_dist = new_dist
            Set FindWithoutQtree = new_item
        End If
    Next new_item
    
    CompLabel.Caption = Format$(AllItems.Count)
End Function

' ************************************************
' Use the quadtree to locate the point.
' ************************************************
Private Function FindWithQtree(X As Single, Y As Single) As QtreeItem
Dim comparisons As Long

    Set FindWithQtree = FindPoint(X, Y, comparisons)
    CompLabel.Caption = Format$(comparisons)
End Function
' ************************************************
' Use the quadtree to locate the point.
' ************************************************
Function FindPoint(X As Single, Y As Single, comparisons As Long) As QtreeItem
Dim leaf As QtreeNode
Dim best_item As QtreeItem
Dim best_dist As Single

    ' See what leaf the point is in.
    Set leaf = Root.LocateLeaf( _
        X, Y, Gxmin, Gxmax, Gymin, Gymax)

    ' Find the closest point within the leaf.
    leaf.NearPointInLeaf _
        X, Y, best_item, best_dist, comparisons

    ' Check nearby leaves.
    Root.CheckNearbyLeaves _
        leaf, X, Y, best_item, best_dist, _
        comparisons, Gxmin, Gxmax, Gymin, Gymax

    Set FindPoint = best_item
End Function

Private Sub Form_Load()
    Gxmin = ScaleLeft
    Gxmax = ScaleWidth + Gxmin
    Gymin = ScaleTop
    Gymax = ScaleHeight + Gymin
    
    MakeItems
    Root.DivideNode Gxmin, Gxmax, Gymin, Gymax
End Sub
Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    SelectPoint X, Y
End Sub

Private Sub Form_Paint()
    Cls
    DrawDots            ' Draw the items.
    DrawQtree           ' Draw the quadtree if appropriate.
    Set Highlighted = Nothing
End Sub

' ************************************************
' Highlight the indicated item.
' ************************************************
Private Sub HighlightItem(new_item As QtreeItem)
Const GAP = 20

Dim X As Single
Dim Y As Single

    DrawMode = vbInvert
    DrawWidth = 3
    
    If Not (Highlighted Is Nothing) Then
        X = Highlighted.X - GAP
        Y = Highlighted.Y - GAP
        Line (X, Y)-Step(2 * GAP, 2 * GAP), , BF
    End If
    
    Set Highlighted = new_item
    If Not (Highlighted Is Nothing) Then
        X = Highlighted.X - GAP
        Y = Highlighted.Y - GAP
        Line (X, Y)-Step(2 * GAP, 2 * GAP), , BF
    End If

    DrawMode = vbCopyPen
    DrawWidth = 1
End Sub

' ************************************************
' Use a strange attractor to make random items to
' put in the quadtree.
' ************************************************
Private Sub MakeItems()
Const a = 2.24
Const b = 0.43
Const c = -0.65
Const d = -2.43
Const e = 1
Const xmax = 2#
Const xmin = -2#
Const ymax = 2#
Const ymin = -2#
Dim Dx As Double
Dim Dy As Double
Dim xtmp As Double
Dim ytmp As Double
Dim ztmp As Double
Dim X As Double
Dim Y As Double
Dim z As Double

Dim num_items As Long
Dim num_str As String
Dim i As Long
Dim new_item As QtreeItem

    num_str = InputBox$("Number of items to create:", "Qtree", "1000")
    If num_str = "" Then    ' The user pressed Cancel.
        End
    ElseIf IsNumeric(num_str) Then
        num_items = CLng(num_str)
    Else
        num_items = 10000
    End If
    
    Dx = (Gxmax - Gxmin) / (xmax - xmin)
    Dy = (Gymax - Gymin) / (ymax - ymin)

    For i = 1 To num_items
        xtmp = Sin(a * Y) - z * Cos(b * X)
        ytmp = z * Sin(c * X) - Cos(d * Y)
        z = e * Sin(X)
        X = xtmp
        Y = ytmp
        Set new_item = New QtreeItem
        new_item.X = Int((X - xmin) * Dx)
        new_item.Y = Int((Y - ymin) * Dy)
        AllItems.Add new_item
        Root.Items.Add new_item
    Next i
End Sub

Private Sub mnuFileExit_Click()
    End
End Sub

' ************************************************
' Present help for this program.
' ************************************************
Private Sub mnuHelpAbout_Click()
Dim txt As String
    
    txt = "This program demonstrates a quadtree." & vbCrLf & vbCrLf
    txt = txt & "Click anywhere on the form and the program will locate the dot nearest to the point you clicked." & vbCrLf & vbCrLf
    txt = txt & "Use the Options menu to enable and disable the quadtree. When the Use Quadtree choice is not checked, the program locates items using an exhaustive search of all of the items created."
    
    MsgBox txt, vbOKOnly, "Help About ... Qtree"
End Sub

Private Sub mnuOptUseQtree_Click()
    mnuOptUseQtree.Checked = Not mnuOptUseQtree.Checked
    Refresh
End Sub

' ************************************************
' Locate and highlight the item nearest to where
' the user clicked.
' ************************************************
Private Sub SelectPoint(X As Single, Y As Single)
Dim new_item As QtreeItem

    ' Find the item selected.
    If mnuOptUseQtree.Checked Then
        Set new_item = FindWithQtree(X, Y)
    Else
        Set new_item = FindWithoutQtree(X, Y)
    End If
    HighlightItem new_item
End Sub

