VERSION 4.00
Begin VB.Form QtreeForm2 
   Appearance      =   0  'Flat
   BackColor       =   &H00000000&
   Caption         =   "Qtree2"
   ClientHeight    =   5475
   ClientLeft      =   1710
   ClientTop       =   870
   ClientWidth     =   5475
   BeginProperty Font {0BE35203-8F91-11CE-9DE3-00AA004BB851} 
      Name            =   "MS Sans Serif"
      Size            =   8.25
      Charset         =   0
      Weight          =   700
      Underline       =   0   'False
      Italic          =   0   'False
      Strikethrough   =   0   'False
   EndProperty
   ForeColor       =   &H000000FF&
   Height          =   6165
   Icon            =   "Qtree2.frx":0000
   Left            =   1650
   LinkTopic       =   "Form1"
   ScaleHeight     =   5475
   ScaleWidth      =   5475
   Top             =   240
   Width           =   5595
   Begin VB.Label CompLabel 
      BorderStyle     =   1  'Fixed Single
      Height          =   255
      Left            =   960
      TabIndex        =   1
      Top             =   30
      Width           =   615
   End
   Begin VB.Label Label1 
      Appearance      =   0  'Flat
      BackColor       =   &H00000000&
      Caption         =   "Examined:"
      ForeColor       =   &H00FFFFFF&
      Height          =   255
      Left            =   90
      TabIndex        =   0
      Top             =   30
      Width           =   975
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileExit 
         Caption         =   "E&xit"
      End
   End
   Begin VB.Menu mnuOpt 
      Caption         =   "&Options"
      Begin VB.Menu mnuOptUseQtree 
         Caption         =   "&Use Quadtree"
         Checked         =   -1  'True
      End
   End
   Begin VB.Menu mnuHelp 
      Caption         =   "&Help"
      Begin VB.Menu mnuHelpAbout 
         Caption         =   "&About This Program..."
      End
   End
End
Attribute VB_Name = "QtreeForm2"
Attribute VB_Creatable = False
Attribute VB_Exposed = False
' ************************************************
' Qtree2.FRM
'
' Main form for pointer faking quadtree program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Dim Highlighted As Long

' ************************************************
' Divide the root node into a quadtree.
' ************************************************
Private Sub BuildQtree()
    DivideNode 1, Gxmin, Gxmax, Gymin, Gymax
End Sub

' ************************************************
' If this quadtree node has children, draw
' dividers between the children.
' ************************************************
Private Sub DisplayQtree(node As Long, xmin As Single, xmax As Single, ymin As Single, ymax As Single)
Dim xmid As Single
Dim ymid As Single
    
    ' If the node has not been divided, do nothing.
    If QtreeNodes(node).SWchild = END_OF_LIST Then Exit Sub
    
    ' Draw the dividers.
    xmid = (xmin + xmax) / 2
    ymid = (ymin + ymax) / 2
    Line (xmin, ymid)-(xmax, ymid)
    Line (xmid, ymin)-(xmid, ymax)

    ' Recursively display the children.
    DisplayQtree QtreeNodes(node).NWchild, xmin, xmid, ymin, ymid
    DisplayQtree QtreeNodes(node).NEchild, xmid, xmax, ymin, ymid
    DisplayQtree QtreeNodes(node).SWchild, xmin, xmid, ymid, ymax
    DisplayQtree QtreeNodes(node).SEchild, xmid, xmax, ymid, ymax
End Sub

' ************************************************
' Draw a dot for each item.
' ************************************************
Private Sub DrawDots()
Dim i As Long

    For i = MIN_ITEM To NextItem - 1
        PSet (Items(i).X, Items(i).Y), vbRed
    Next i
End Sub

' ************************************************
' If the Use Quadtree option is checked, display
' the quadtree.
' ************************************************
Private Sub DrawQtree()
    If Not mnuOptUseQtree.Checked Then Exit Sub

    DisplayQtree 1, Gxmin, Gxmax, Gymin, Gymax
End Sub

' ************************************************
' Locate the point the slow way.
' ************************************************
Private Function FindWithoutQtree(X As Single, Y As Single) As Long
Dim i As Long
Dim Dx As Single
Dim Dy As Single
Dim best_dist As Single
Dim new_dist As Single

    Dx = Items(MIN_ITEM).X - X
    Dy = Items(MIN_ITEM).Y - Y
    best_dist = Dx * Dx + Dy * Dy
    FindWithoutQtree = MIN_ITEM

    For i = MIN_ITEM + 1 To NextItem - 1
        Dx = Items(i).X - X
        Dy = Items(i).Y - Y
        new_dist = Dx * Dx + Dy * Dy
        If new_dist < best_dist Then
            best_dist = new_dist
            FindWithoutQtree = i
        End If
    Next i
    CompLabel.Caption = Format$(NextItem - MIN_ITEM)
End Function

' ************************************************
' Use the quadtree to locate the point.
' ************************************************
Private Function FindWithQtree(X As Single, Y As Single) As Long
Dim comparisons As Long

    FindWithQtree = FindPoint(X, Y, comparisons)
    CompLabel.Caption = Format$(comparisons)
End Function

Private Sub Form_Load()
    Highlighted = END_OF_LIST
    QtreeInitialize

    Gxmin = ScaleLeft
    Gxmax = ScaleWidth + Gxmin
    Gymin = ScaleTop
    Gymax = ScaleHeight + Gymin
    
    MakeItems
    BuildQtree
End Sub

Private Sub Form_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    SelectPoint X, Y
End Sub

Private Sub Form_Paint()
    Cls
    DrawDots            ' Draw the items.
    DrawQtree           ' Draw the quadtree if appropriate.
    HighlightItem END_OF_LIST   ' Rehighlight the previous item.
End Sub

' ************************************************
' Highlight the indicated item.
'
' If item <=0 then just rehighlight the already
' selected item.
' ************************************************
Private Sub HighlightItem(item As Long)
Const GAP = 20

Dim X As Single
Dim Y As Single

    DrawMode = 6
    DrawWidth = 3
    
    If Highlighted <> END_OF_LIST And item <> END_OF_LIST Then
        X = Items(Highlighted).X - GAP
        Y = Items(Highlighted).Y - GAP
        Line (X, Y)-Step(2 * GAP, 2 * GAP), , B
    End If
    
    If item <> END_OF_LIST Then Highlighted = item
    
    If Highlighted <> END_OF_LIST Then
        X = Items(Highlighted).X - GAP
        Y = Items(Highlighted).Y - GAP
        Line (X, Y)-Step(2 * GAP, 2 * GAP), , B
    End If

    DrawMode = 13
    DrawWidth = 1
End Sub

' ************************************************
' Use a strange attractor to make random items to
' put in the quadtree.
' ************************************************
Private Sub MakeItems()
Const a = 2.24
Const b = 0.43
Const c = -0.65
Const d = -2.43
Const e = 1
Const xmax = 2#
Const xmin = -2#
Const ymax = 2#
Const ymin = -2#
Dim Dx As Double
Dim Dy As Double
Dim xtmp As Double
Dim ytmp As Double
Dim ztmp As Double
Dim X As Double
Dim Y As Double
Dim z As Double

Dim num_items As Long
Dim num_str As String
Dim i As Long
Dim item As Long

    num_str = InputBox$("Number of items to create:", "Qtree2", "1000")
    If num_str = "" Then    ' The user pressed Cancel.
        End
    ElseIf IsNumeric(num_str) Then
        num_items = CLng(num_str)
    Else
        num_items = 10000
    End If
    
    Dx = (Gxmax - Gxmin) / (xmax - xmin)
    Dy = (Gymax - Gymin) / (ymax - ymin)

    For i = 1 To num_items
        item = NewItem()
        xtmp = Sin(a * Y) - z * Cos(b * X)
        ytmp = z * Sin(c * X) - Cos(d * Y)
        z = e * Sin(X)
        X = xtmp
        Y = ytmp
        Items(item).X = Int((X - xmin) * Dx)
        Items(item).Y = Int((Y - ymin) * Dy)
        Items(item).NextItem = QtreeNodes(1).NWchild
        QtreeNodes(1).NWchild = item
        QtreeNodes(1).NumItems = QtreeNodes(1).NumItems + 1
    Next i
End Sub

Private Sub mnuFileExit_Click()
    End
End Sub

' *******************************************************
' Present help for this program.
' *******************************************************
Private Sub mnuHelpAbout_Click()
Dim txt As String
    
    txt = "This program demonstrates a quadtree using pointer faking." & vbCrLf & vbCrLf
    txt = txt & "Click anywhere on the form and the program will locate the item nearest to the point you clicked." & vbCrLf & vbCrLf
    txt = txt & "Use the Options menu to enable and disable the quadtree. When the Use Quadtree choice is not checked, the program locates items using an exhaustive search of all of the items created."
    
    MsgBox txt, vbOKOnly, "Help About ... Qtree2"
End Sub

Private Sub mnuOptUseQtree_Click()
    mnuOptUseQtree.Checked = Not mnuOptUseQtree.Checked
    Refresh
End Sub

' ************************************************
' Locate and highlight the item nearest to where
' the user clicked.
' ************************************************
Private Sub SelectPoint(X As Single, Y As Single)
Dim item As Long

    ' Find the item selected.
    If mnuOptUseQtree.Checked Then
        item = FindWithQtree(X, Y)
    Else
        item = FindWithoutQtree(X, Y)
    End If
    HighlightItem item
End Sub

