Attribute VB_Name = "AVLStuff"
' ************************************************
' AVL.BAS
'
' Source code for AVL tree example program.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Global Const LEFT_HEAVY = -1
Global Const BALANCED = 0
Global Const RIGHT_HEAVY = 1

' ************************************************
' Remove the indicated item from the tree.
' ************************************************
Public Sub DeleteItem(node As AVLNode, txt As String, shrunk As Boolean)
Dim child As AVLNode
Dim target As AVLNode

    If node Is Nothing Then
        Beep
        MsgBox "Item " & txt & " is not in the tree."
        shrunk = False
        Exit Sub
    End If

    If txt < node.Box.Caption Then
        Set child = node.LeftChild
        DeleteItem child, txt, shrunk
        Set node.LeftChild = child
        If shrunk Then RebalanceLeftShrunk node, shrunk
    ElseIf txt > node.Box.Caption Then
        Set child = node.RightChild
        DeleteItem child, txt, shrunk
        Set node.RightChild = child
        If shrunk Then RebalanceRightShrunk node, shrunk
    Else
        Set target = node
        If target.RightChild Is Nothing Then
            ' No children or left child only.
            Set node = target.LeftChild
            shrunk = True
        ElseIf target.LeftChild Is Nothing Then
            ' Right child only.
            Set node = target.RightChild
            shrunk = True
        Else
            ' Two children
            Set child = target.LeftChild
            ReplaceRightmost child, shrunk, target
            Set target.LeftChild = child
            If shrunk Then RebalanceLeftShrunk node, shrunk
        End If
    End If
End Sub

' ************************************************
' Rebalance for the case where the Right branch
' has shrunk.
' ************************************************
Private Sub RebalanceRightShrunk(node As AVLNode, shrunk As Boolean)
Dim child As AVLNode
Dim child_bal As Integer
Dim grandchild As AVLNode
Dim grandchild_bal As Integer

    If node.Balance = RIGHT_HEAVY Then
        ' Was Right heavy, now it is balanced.
        node.Balance = BALANCED
    ElseIf node.Balance = BALANCED Then
        ' Was balanced, now it is Left heavy.
        node.Balance = LEFT_HEAVY
        shrunk = False
    Else
        ' Was Left heavy, now unbalanced.
        Set child = node.LeftChild
        child_bal = child.Balance
        If child_bal <= 0 Then
            ' Right rotation.
            Set node.LeftChild = child.RightChild
            Set child.RightChild = node
            If child_bal = BALANCED Then
                node.Balance = LEFT_HEAVY
                child.Balance = RIGHT_HEAVY
                shrunk = False
            Else
                node.Balance = BALANCED
                child.Balance = BALANCED
            End If
            Set node = child
        Else
            ' Left-right rotation.
            Set grandchild = child.RightChild
            grandchild_bal = grandchild.Balance
            Set child.RightChild = grandchild.LeftChild
            Set grandchild.LeftChild = child
            Set node.LeftChild = grandchild.RightChild
            Set grandchild.RightChild = node
            If grandchild_bal = LEFT_HEAVY Then
                node.Balance = RIGHT_HEAVY
            Else
                node.Balance = BALANCED
            End If
            If grandchild_bal = RIGHT_HEAVY Then
                child.Balance = LEFT_HEAVY
            Else
                child.Balance = BALANCED
            End If
            Set node = grandchild
            grandchild.Balance = BALANCED
        End If
    End If
End Sub
' ************************************************
' Perform left or right-left rotations to
' rebalance the tree after a left branch has
' shrunk.
' ************************************************
Private Sub RebalanceLeftShrunk(node As AVLNode, shrunk As Boolean)
Dim child As AVLNode
Dim child_bal As Integer
Dim grandchild As AVLNode
Dim grandchild_bal As Integer

    If node.Balance = LEFT_HEAVY Then
        ' Was left heavy, now it is balanced.
        node.Balance = BALANCED
    ElseIf node.Balance = BALANCED Then
        ' Was balanced, now it is right heavy.
        node.Balance = RIGHT_HEAVY
        shrunk = False
    Else
        ' Was right heavy, now unbalanced.
        Set child = node.RightChild
        child_bal = child.Balance
        If child_bal >= 0 Then
            ' Left rotation.
            Set node.RightChild = child.LeftChild
            Set child.LeftChild = node
            If child_bal = BALANCED Then
                node.Balance = RIGHT_HEAVY
                child.Balance = LEFT_HEAVY
                shrunk = False
            Else
                node.Balance = BALANCED
                child.Balance = BALANCED
            End If
            Set node = child
        Else
            ' Right-left rotation.
            Set grandchild = child.LeftChild
            grandchild_bal = grandchild.Balance
            Set child.LeftChild = grandchild.RightChild
            Set grandchild.RightChild = child
            Set node.RightChild = grandchild.LeftChild
            Set grandchild.LeftChild = node
            If grandchild_bal = RIGHT_HEAVY Then
                node.Balance = LEFT_HEAVY
            Else
                node.Balance = BALANCED
            End If
            If grandchild_bal = LEFT_HEAVY Then
                child.Balance = RIGHT_HEAVY
            Else
                child.Balance = BALANCED
            End If
            Set node = grandchild
            grandchild.Balance = BALANCED
        End If
    End If
End Sub

' ************************************************
' Perform a right or left-right rotation to
' rebalance this node.
' ************************************************
Private Sub RebalanceLeftGrew(parent As AVLNode)
Dim child As AVLNode
Dim grandchild As AVLNode

    Set child = parent.LeftChild
            
    If child.Balance = LEFT_HEAVY Then
        ' Perform a right rotation.
        Set parent.LeftChild = child.RightChild
        Set child.RightChild = parent
        parent.Balance = BALANCED
        Set parent = child
    Else
        ' Perform a left-right rotation.
        Set grandchild = child.RightChild
        Set child.RightChild = grandchild.LeftChild
        Set grandchild.LeftChild = child
        Set parent.LeftChild = grandchild.RightChild
        Set grandchild.RightChild = parent
        If grandchild.Balance = LEFT_HEAVY Then
            parent.Balance = RIGHT_HEAVY
        Else
            parent.Balance = BALANCED
        End If
        If grandchild.Balance = RIGHT_HEAVY Then
            child.Balance = LEFT_HEAVY
        Else
            child.Balance = BALANCED
        End If
        Set parent = grandchild
    End If  ' End if (right rotation)... else (left-right rotation)
    parent.Balance = BALANCED
End Sub
' ************************************************
' Perform a left or right-left rotation to
' rebalance this node.
' ************************************************
Private Sub RebalanceRightGrew(parent As AVLNode)
Dim child As AVLNode
Dim grandchild As AVLNode

    Set child = parent.RightChild

    If child.Balance = RIGHT_HEAVY Then
        ' Perform a left rotation.
        Set parent.RightChild = child.LeftChild
        Set child.LeftChild = parent
        parent.Balance = BALANCED
        Set parent = child
    Else
        ' Perform a right-left rotation.
        Set grandchild = child.LeftChild
        Set child.LeftChild = grandchild.RightChild
        Set grandchild.RightChild = child
        Set parent.RightChild = grandchild.LeftChild
        Set grandchild.LeftChild = parent
        If grandchild.Balance = RIGHT_HEAVY Then
            parent.Balance = LEFT_HEAVY
        Else
            parent.Balance = BALANCED
        End If
        If grandchild.Balance = LEFT_HEAVY Then
            child.Balance = RIGHT_HEAVY
        Else
            child.Balance = BALANCED
        End If
        Set parent = grandchild
    End If  ' End if (right rotation)... else (double right rotation)
    parent.Balance = BALANCED
End Sub

' ************************************************
' Replace the target node with the rightmost node
' to the left of it.
' ************************************************
Private Sub ReplaceRightmost(repl As AVLNode, shrunk As Boolean, target As AVLNode)
Dim child As AVLNode

    If repl.RightChild Is Nothing Then
        target.Box.Caption = repl.Box.Caption
        Set target = repl
        Set repl = repl.LeftChild
        shrunk = True
    Else
        Set child = repl.RightChild
        ReplaceRightmost child, shrunk, target
        Set repl.RightChild = child
        If shrunk Then RebalanceRightShrunk repl, shrunk
    End If
End Sub

' ************************************************
' Insert a new item into the tree.
'
' Set has_grown = True if the subtree rooted at
' this node grew taller. This will help us as we
' back up the tree as the recursion ends. It will
' allow us to determine whether we need to
' rebalance farther up the tree.
' ************************************************
Public Sub InsertItem(node As AVLNode, parent As AVLNode, txt As String, has_grown As Boolean)
Dim child As AVLNode

    ' If this is the bottom of the tree, leave
    ' the parent pointing to the new node.
    If parent Is Nothing Then
        Set parent = node
        parent.Balance = BALANCED
        has_grown = True
        Exit Sub
    End If
    
    ' Continue down the left or right subtree.
    If txt <= parent.Box.Caption Then
        ' Insert the child in the left subtree
        Set child = parent.LeftChild
        InsertItem node, child, txt, has_grown
        Set parent.LeftChild = child

        ' See if rebalancing is necessary. It will
        ' not be if the addition never unbalanced
        ' the tree or if we rebalanced the tree at
        ' a deeper level of recursion. In either
        ' case has_grown will be False.
        If Not has_grown Then Exit Sub
        
        If parent.Balance = RIGHT_HEAVY Then
            ' Was right heavy, now balanced. This
            ' subtree did not grow so the tree is
            ' balanced.
            parent.Balance = BALANCED
            has_grown = False
        ElseIf parent.Balance = BALANCED Then
            ' Was balanced, now left heavy. This
            ' subtree is still balanced but it
            ' grew so we need to continue checking
            ' up the tree.
            parent.Balance = LEFT_HEAVY
        Else
            ' Was left heavy, now left unbalanced.
            ' Perform a rotation to rebalance at
            ' this node.
            RebalanceLeftGrew parent
            has_grown = False
        End If  ' End checking this node's balance.
    Else
        ' Insert the child in the right subtree
        Set child = parent.RightChild
        InsertItem node, child, txt, has_grown
        Set parent.RightChild = child

        ' See if rebalancing is necessary. It will
        ' not be if the addition never unbalanced
        ' the tree or if we rebalanced the tree at
        ' a deeper level of recursion. In either
        ' case has_grown will be False.
        If Not has_grown Then Exit Sub
        
        If parent.Balance = LEFT_HEAVY Then
            ' Was left heavy, now balanced. This
            ' subtree did not grow so the tree is
            ' balanced.
            parent.Balance = BALANCED
            has_grown = False
        ElseIf parent.Balance = BALANCED Then
            ' Was balanced, now right heavy. This
            ' subtree is still balanced but it
            ' grew so we need to continue checking
            ' up the tree.
            parent.Balance = RIGHT_HEAVY
        Else
            ' Was right heavy, now right unbalanced.
            ' Perform a rotation to rebalance at
            ' this node.
            RebalanceRightGrew parent
            has_grown = False
        End If  ' End checking this node's balance.
    End If  ' End if (down left) ... else (down right)
End Sub

