Attribute VB_Name = "BucketsortStuff"
' ************************************************
' Bucket.BAS
'
' Source code for bucketsort.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

Dim NextInList() As Long

' ************************************************
' Array-based BucketSort.
' ************************************************
Public Sub ArrayBucketSort(List() As Long, Scratch() As Long, min As Long, max As Long, NumBuckets As Long)
Dim counts() As Long
Dim offsets() As Long

Dim i As Long
Dim Value As Long
Dim min_value As Long
Dim max_value As Long
Dim value_scale As Double
Dim bucket_num As Long
Dim next_spot As Long
Dim num_in_bucket As Long

    ' If the list has no more than CutOff elements,
    ' finish it off with SelectionSort.
    If max - min + 1 < CutOff Then
        Selectionsort List(), min, max
        Exit Sub
    End If
    
    ' Find the min and max values.
    min_value = List(min)
    max_value = min_value
    For i = min + 1 To max
        Value = List(i)
        If min_value > Value Then min_value = Value
        If max_value < Value Then max_value = Value
    Next i

    ' If min_value = max_value, there is only one
    ' value so the list is sorted.
    If min_value = max_value Then Exit Sub
    
    ' Create the empty bucket counts array.
    ReDim counts(1 To NumBuckets)
    
    value_scale = _
        CDbl(NumBuckets - 1) / _
        CDbl(max_value - min_value)
    
    ' Create bucket counts.
    For i = min To max
        If List(i) = max_value Then
            bucket_num = NumBuckets
        Else
            bucket_num = _
                Int((List(i) - min_value) * _
                    value_scale) + 1
        End If
        counts(bucket_num) = counts(bucket_num) + 1
    Next i

    ' Turn the counts into offsets.
    ReDim offsets(1 To NumBuckets)
    next_spot = min
    For i = 1 To NumBuckets
        offsets(i) = next_spot
        next_spot = next_spot + counts(i)
    Next i

    ' Place the values in their buckets.
    For i = min To max
        If List(i) = max_value Then
            bucket_num = NumBuckets
        Else
            bucket_num = _
                Int((List(i) - min_value) * _
                    value_scale) + 1
        End If
        Scratch(offsets(bucket_num)) = List(i)
        offsets(bucket_num) = offsets(bucket_num) + 1
    Next i

    ' Recursively sort the buckets that hold
    ' more than one item.
    next_spot = min
    For i = 1 To NumBuckets
        If counts(i) > 1 Then ArrayBucketSort _
            Scratch(), List(), next_spot, _
            next_spot + counts(i) - 1, counts(i)
        next_spot = next_spot + counts(i)
    Next i
    
    ' Copy the scratch array back into the list.
    For i = min To max
        List(i) = Scratch(i)
    Next i
End Sub

' ************************************************
' Linked list bucket sort
' ************************************************
Public Sub LinkBucketSort(ListTop As ListCell)
Dim count As Long
Dim min_value As Long
Dim max_value As Long
Dim Value As Long
Dim item As ListCell
Dim nxt As ListCell
Dim bucket() As New ListCell
Dim value_scale As Double
Dim bucket_num As Long
Dim i As Long

    Set item = ListTop.NextCell
    If item Is Nothing Then Exit Sub
    
    ' Count the items and find the min and max values.
    count = 1
    min_value = item.Value
    max_value = min_value
    Set item = item.NextCell
    Do While Not (item Is Nothing)
        count = count + 1
        Value = item.Value
        If min_value > Value Then min_value = Value
        If max_value < Value Then max_value = Value
        Set item = item.NextCell
    Loop

    ' If min_value = max_value, there is only one
    ' value so the list is sorted.
    If min_value = max_value Then Exit Sub
    
    ' If the list has no more than CutOff elements,
    ' finish it off with LinkInsertionSort.
    If count <= CutOff Then
        LinkInsertionSort ListTop
        Exit Sub
    End If
    
    ' Allocate the empty buckets.
    ReDim bucket(1 To count)
    
    value_scale = _
        CDbl(count - 1) / _
        CDbl(max_value - min_value)
    
    ' Move the items into the buckets.
    Set item = ListTop.NextCell
    Do While Not (item Is Nothing)
        Set nxt = item.NextCell
        Value = item.Value
        If Value = max_value Then
            bucket_num = count
        Else
            bucket_num = _
                Int((Value - min_value) * _
                    value_scale) + 1
        End If
        Set item.NextCell = bucket(bucket_num).NextCell
        Set bucket(bucket_num).NextCell = item
        Set item = nxt
    Loop

    ' Recursively sort the buckets that hold more
    ' than one item.
    For i = 1 To count
        If Not (bucket(i).NextCell Is Nothing) Then _
            LinkBucketSort bucket(i)
    Next i
    
    ' Merge the sorted lists.
    Set ListTop.NextCell = bucket(count).NextCell
    For i = count - 1 To 1 Step -1
        Set item = bucket(i).NextCell
        If Not (item Is Nothing) Then
            Do While Not (item.NextCell Is Nothing)
                Set item = item.NextCell
            Loop
            Set item.NextCell = ListTop.NextCell
            Set ListTop.NextCell = bucket(i).NextCell
        End If
    Next i
End Sub

