Attribute VB_Name = "InsertionsortStuff"
' ************************************************
' FInsert.BAS
'
' Source code for insertionsort using MemCopy.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit
' ************************************************
' Use interpolation search to see where the item
' belongs in the list.
'
' This routine is used by subroutine
' yInterstionsort which is not used by the
' program. They are included because they are
' easier to understand than the routine that is
' used. That routine includes the interpolation
' code within itself for greater speed.
' ************************************************
Private Function FindSpot(target As Long, List() As Long, list_min As Long, list_max As Long) As Long
Dim max As Long
Dim min As Long
Dim middle As Long

    ' At all times, the correct position will be
    ' between min and max: min <= pos <= max.
    min = list_min
    max = list_max
    Do While min <= max
        ' Don't divide by zero.
        If List(min) = List(max) Then
            If List(min) >= target Then
                FindSpot = min
            Else
                FindSpot = max + 1
            End If
            Exit Function
        End If
        
        ' Compute the dividing point.
        middle = min + (target - List(min)) * ((max - min) / (CDbl(List(max)) - List(min)))
        
        ' Make sure we stay in bounds.
        If middle < min Then middle = min
        If middle > max Then middle = max
        
        If target = List(middle) Then ' We found it.
            FindSpot = middle
            Exit Function
        ElseIf target < List(middle) Then ' Search the left half.
            max = middle - 1
        Else ' Search the right half.
            min = middle + 1
        End If
    Loop
    
    ' At this point max <= min.
    If max < list_min Then
        max = list_min
    ElseIf min > list_max Then
        min = list_max
    End If

    If List(max) >= target Then
        FindSpot = max
    ElseIf List(min) >= target Then
        FindSpot = min
    Else
        FindSpot = min + 1
    End If
End Function
' ************************************************
' Insertionsort using MemCopy and interpolated
' search for the next item's position.
' ************************************************
Public Sub Insertionsort(List() As Long, min As Long, max As Long)
Dim i As Long
Dim j As Long
Dim max_sorted As Long
Dim next_num As Long
Dim xmax As Long
Dim xmin As Long
Dim middle As Long

    max_sorted = min - 1
    For i = min To max
        ' This is the number we are inserting.
        next_num = List(i)
        
        ' See where it belongs in the list.
        ' At all times, the correct position will be
        ' between xmin and xmax: xmin <= pos <= xmax.
        j = min - 1
        xmin = min
        xmax = max_sorted
        Do While xmin <= xmax
            ' Don't divide by zero.
            If List(xmin) = List(xmax) Then
                If List(xmin) >= next_num Then
                    j = xmin
                Else
                    j = xmax + 1
                End If
                Exit Do
            End If
            
            ' Compute the dividing point.
            middle = xmin + (next_num - List(xmin)) * ((xmax - xmin) / (CDbl(List(xmax)) - List(xmin)))
            
            ' Make sure we stay in bounds.
            If middle < xmin Then middle = xmin
            If middle > xmax Then middle = xmax
            
            If next_num = List(middle) Then ' We found it.
                j = middle
                Exit Do
            ElseIf next_num < List(middle) Then ' Search the left half.
                xmax = middle - 1
            Else ' Search the right half.
                xmin = middle + 1
            End If
        Loop
        
        If j < min Then
            ' At this point xmax <= xmin.
            If xmax < min Then
                xmax = min
            ElseIf xmin > max Then
                xmin = max
            End If
        
            If List(xmax) >= next_num Then
                j = xmax
            ElseIf List(xmin) >= next_num Then
                j = xmin
            Else
                j = xmin + 1
            End If
        End If

        ' Bump the bigger sorted numbers down to
        ' make room for the new number.
        If max_sorted >= j Then _
            MemCopy List(j + 1), List(j), _
                Len(next_num) * (max_sorted - j + 1)

        ' Insert the new number.
        List(j) = next_num
        
        ' Increment the count of the sorted items.
        max_sorted = max_sorted + 1
    Next i
End Sub
' ************************************************
' Insertionsort using MemCopy and interpolated
' search for the next item's position.
'
' This routine is not used by the program. It is
' included because it is easier to understand than
' the routine that is used. That routine includes
' the interpolation code within itself to avoid
' making calls to function FindSpot.
' ************************************************
Public Sub yInsertionsort(List() As Long, min As Long, max As Long)
Dim i As Long
Dim j As Long
Dim num_sorted As Long
Dim next_num As Long

    num_sorted = 0
    For i = min To max
        ' This is the number we are inserting.
        next_num = List(i)
        
        ' See where it belongs in the list.
        j = FindSpot(next_num, List(), _
            min, num_sorted)

        ' Bump the bigger sorted numbers down to
        ' make room for the new number.
        If num_sorted >= j Then _
            MemCopy List(j + 1), List(j), _
                Len(next_num) * (num_sorted - j + 1)

        ' Insert the new number.
        List(j) = next_num
        
        ' Increment the count of the sorted items.
        num_sorted = num_sorted + 1
    Next i
End Sub

' ************************************************
' Insertionsort without interpolated search for
' the next item's position.
'
' This routine is not used by the program. It is
' provided so you can compare it to the version
' that does not use MemCopy so you can see how big
' a difference MemCopy makes.
' ************************************************
Public Sub xInsertionsort(List() As Long, min As Long, max As Long)
Dim i As Long
Dim j As Long
Dim k As Long
Dim num_sorted As Long
Dim next_num As Long

    num_sorted = 0
    For i = min To max
        ' This is the number we are inserting.
        next_num = List(i)
        
        ' See where it belongs in the list.
        For j = min To min + num_sorted - 1
            If List(j) >= next_num Then Exit For
        Next j
        
        ' Bump the bigger sorted numbers down to make
        ' room for the new number.
        If num_sorted >= j Then _
            MemCopy List(j + 1), List(j), _
                Len(next_num) * (num_sorted - j + 1)

        ' Insert the new number.
        List(j) = next_num
        
        ' Increment the count of the sorted items.
        num_sorted = num_sorted + 1
    Next i
End Sub
