Attribute VB_Name = "HeapsortStuff"
' ************************************************
' Heap.BAS
'
' Source code for heapsort.
' ************************************************
' Copyright (C) 1997 John Wiley & Sons, Inc.
' All rights reserved. See additional copyright
' information in RIGHTS.TXT.
' ************************************************
Option Explicit

' ************************************************
' Push the top item down through the heap until it
' can go no farther.
' ************************************************
Private Sub HeapPushDown(List() As Long, ByVal min As Long, ByVal max As Long)
Dim tmp As Long
Dim j As Long

    tmp = List(min)
    Do
        j = 2 * min
        If j <= max Then
            ' Make j point to the larger of the children.
            If j < max Then
                If List(j + 1) > List(j) Then _
                    j = j + 1
            End If

            If List(j) > tmp Then
                ' A child is bigger. Swap with the child.
                List(min) = List(j)
                ' Push down beneath that child.
                min = j
            Else
                ' The parent is bigger. We're done.
                Exit Do
            End If
        Else
            Exit Do
        End If
    Loop
    List(min) = tmp
End Sub

' ************************************************
' Heapsort.
' ************************************************
Public Sub Heapsort(List() As Long, ByVal min As Long, ByVal max As Long)
Dim i As Long
Dim tmp As Long

    ' Make a heap (except for the root node).
    For i = (max + min) \ 2 To min + 1 Step -1
        HeapPushDown List(), i, max
    Next i

    ' Repeatedly:
    '   1. HeapPushDown.
    '   2. Output the root.
    For i = max To min + 1 Step -1
        ' HeapPushDown.
        HeapPushDown List(), min, i

        ' Output the root.
        tmp = List(min)
        List(min) = List(i)
        List(i) = tmp
    Next i
End Sub

